local ProgressLayer    = class("ProgressLayer")
ProgressLayer._name = "ProgressLayer"
ProgressLayer._bar     = nil
ProgressLayer._tips    = nil
ProgressLayer._count   = 0
ProgressLayer._callback= nil
ProgressLayer._value   = 0
ProgressLayer._nowvalue= 0
ProgressLayer._str     = ""
ProgressLayer._lasttick= 0
ProgressLayer._back = 0
ProgressLayer._privacty = 0
ProgressLayer._google = ""
ProgressLayer._alias  = ""
ProgressLayer._action = nil

function ProgressLayer.extend(target)
    local t = tolua.getpeer(target)
    if not t then
        t = {}
        tolua.setpeer(target, t)
    end
    setmetatable(t, ProgressLayer)
    return target
end


function ProgressLayer:setProgress(value)
	if value < 1 then
		value = 1
	end
	if value > 100 then
		value = 100
	end
    self._value = value
    if value <= 1 then
        self._nowvalue = 0
    end
    platform_setLoadingBarPercent(value * 10)
end
function ProgressLayer:setTips(tips)
    if tips ~= nil then
        if cc.PLATFORM_OS_WINDOWS == cc.Application:getInstance():getTargetPlatform() then
            self:getChildByName("tips"):setVisible(true)
            self:getChildByName("tips"):setString(tips)
        else
            platform_setLoadingBarTip(tips)
        end
    end
end
function ProgressLayer:setLabelColor(color)
    self:getChildByName("tips"):setColor(cc.c3b(255,0,0))
    platform_setLoadingBarTipColor(color)
end

function ProgressLayer:beginJsons(callback,tips)
    self._tips = tips
    local count = ff.gameCore:addAsyncJsonStart()
    if count > 0 then
        if callback ~= nil then
            self:setTips(tips.."...0/"..count)       
        end    
        self._count    = count
        self._total    = count
        self._callback = callback
    else
        if callback ~= nil then
            callback()
        end    
    end    
end
function ProgressLayer:showTop()
    local lay = cc.LayerColor:create(cc.c4b(255,0,0,150))    
    cc.Director:getInstance():setNotificationNode(lay)
    local notif = cc.Director:getInstance():getNotificationNode()
    notif:setTag(999)
end

function ProgressLayer:finish()
    self._count = self._count - 1
    if self._tips ~= nil then
        self:setTips(self._tips.."..."..(self._total - self._count).."/"..self._total)       
    end
    cclog("count:"..self._count)
    if self._count == 0 then
        if self._callback ~= nil then
            self._callback()
        end    
    end
end
function ProgressLayer:onShowAsync(callback,tips)
    self._callback = callback
    if self._total == nil or self._total <= 0 then
        self._callback()
    else
        self._tips = tips
        if self._tips ~= nil then
            self:setTips(self._tips.."..."..(self._total - self._count).."/"..self._total)       
        end
        if self._count == 0 then
            if self._callback ~= nil then
                self._callback()
            end    
        end    
    end    
end
function ProgressLayer:onLogin()
    if ff.restart == false then
        dd.initCfg()
    end
    performWithDelay(dd.loadingLayer, login_begin, 0.05)
end   

function ProgressLayer.onSuccess()
    if cc.PLATFORM_OS_ANDROID == cc.Application:getInstance():getTargetPlatform() then
        if ff.dbmanager:create("data"..ff.db_version..".db") == 0 then
            ff.loadingLayer:setProgress(0)
            downDB(ff.loadingLayer.onSuccess,ff.loadingLayer.onError)
            return
        end
    else
        if ff.dbmanager:create("data.db") == 0 then
            downDB(ff.loadingLayer.onSuccess,ff.loadingLayer.onError)
            return
        end
    end

    if ff.needUpdate == true then
        updateVersionSQL()
    end


    --初始化
    require "define"
    dd.currentLayer = ff.loadingLayer 
    dd.loadingLayer = ff.loadingLayer     
    dd.initGame()

    platform_showLoadingBar(2)
    platform_setLoadingBarTip2(ff.langString("初始化UI..."))
    dd.loadingLayer:setProgress(0)
    if dd.loadingLayer ~= nil then
        dd.loadingLayer:setProgress(20)
        dd.loadingLayer:setTips(ff.langString("开始连接服务器..."))
    end    
    dd.loadingLayer:onLogin()
end
function ProgressLayer.onError(errorCode)
    if ff.loadingLayer ~= nil then
        ff.loadingLayer.onSuccess()
    end
end   
-- 初始化各种
function ProgressLayer.checkUpdate()
    if ff.show_servers == false then
        platform_showLoadingBar(1)
        ff.loadingLayer:setTips(ff.langString("开始连接服务器..."))
        platform_setLoadingBarPercent(1 * 10)

        if string.len(ff.loadingLayer._google) >= 32 then
            --cclog("google:"..ff.loadingLayer._google)        
            cc.UserDefault:getInstance():setStringForKey("google",ff.loadingLayer._google)
            cc.UserDefault:getInstance():setStringForKey("login_alias",ff.loadingLayer._alias)
        end
        local result = hif.checkLogin()
        if result == false then
            platform_showLoadingBar(99)
            platform_setLoadingBarTipColor(cc.c3b(255,0,0))                
            ff.loadingLayer:setTips(ff.langString("服务忙，请稍后再试。"))
            platform_setLoadingBarPercent(1 * 10)
            return
        end
    end
    if ff.loadingLayer ~= nil then
        platform_showLoadingBar(1)
        ff.loadingLayer:setProgress(5)
        ff.loadingLayer:setTips(ff.langString("Check Update..."))
    end    
    --版本更新
    if cc.PLATFORM_OS_ANDROID == cc.Application:getInstance():getTargetPlatform() then
        checkVersion(ff.loadingLayer.onSuccess,ff.loadingLayer.onError)
    else
        ff.loadingLayer.onSuccess()
    end
end

function continueGame(ret)
    performWithDelay(ff.loadingLayer, ff.loadingLayer.continue, 0.2)
end

function ProgressLayer.continue()
    platform_showFull()
    if ff.loadingLayer ~= nil then
        ff.loadingLayer:onStart()
    end
end

function ProgressLayer:start()
    local function checkCallBack()
        if self._back > 0 then
            --cclog("check ok")
            self:stopAction(self._action)
            
            local id = hif._lists:getLong(self._back,"id")
            local index = 1    
            for i=1,hif._lists:rowCount() do
                if hif._lists:getLong(index,"id") ~= id then
                    hif._lists:delRow(index)
                else
                    index = index + 1
                end
            end
            platform_showLoadingBar(1)
            ff.loadingLayer:setTips(ff.langString("开始连接服务器..."))
            platform_setLoadingBarPercent(1 * 10)

            hif._row = 1     
            if string.len(self._google) >= 32 then
                --cclog("google:"..self._google)        
                cc.UserDefault:getInstance():setStringForKey("google",self._google)
                cc.UserDefault:getInstance():setStringForKey("login_alias",self._alias)
            end
            local result = hif.checkLogin()
            if result == false then
                platform_showLoadingBar(99)
                platform_setLoadingBarTipColor(cc.c3b(255,0,0))                
                ff.loadingLayer:setTips(ff.langString("服务忙，请稍后再试。"))
                platform_setLoadingBarPercent(1 * 10)
                return
            end
            performWithDelay(self, self.checkUpdate, 0.1)
        end
    end
    local function checkCallBack2()
        if self._privacty > 0 then
            --cclog("check ok")
            self:stopAction(self._action)
            cc.UserDefault:getInstance():setIntegerForKey("privacy",1)
           local index = 1
            for i=1,hif._lists:rowCount() do
                if hif._lists:getLong(index,"type") ~= 2 then
                    hif._lists:delRow(index)
                else
                    index = index + 1
                end
            end
            
            performWithDelay(self, self.checkUpdate, 0.1)
        end
    end
    local privacy = cc.UserDefault:getInstance():getIntegerForKey("privacy")
    if cc.PLATFORM_OS_WINDOWS == cc.Application:getInstance():getTargetPlatform() then
        privacy = 1
    end
    
    if ff.show_servers == true then
        self._back = 0
        self:showServers()
        self._action = schedule(self, checkCallBack, 1)        
    else    
        if not privacy or privacy == 0 then
            self._privacty = 0
            platform_registerPrivacyCallBack(self.privacy_callback)
            self._action = schedule(self, checkCallBack2, 1)        
        else    
            if cc.PLATFORM_OS_ANDROID == cc.Application:getInstance():getTargetPlatform() then
                local index = 1
                for i=1,hif._lists:rowCount() do
                    if hif._lists:getLong(index,"type") ~= 2 then
                        hif._lists:delRow(index)
                    else
                        index = index + 1
                    end
                end
            end            
            performWithDelay(self,self.checkUpdate,0.1)
        end        
    end
    
end
function ProgressLayer.checkStart()
    ff.loadingLayer:start()
end


function ProgressLayer.checkGPGCallBack()
    if ff.loadingLayer._back > 0 then
        ff.loadingLayer:stopAction(ff.loadingLayer._action)
        ff.loadingLayer._action = nil
        performWithDelay(ff.loadingLayer,ff.loadingLayer.checkStart,0.1)
    end
end

function ProgressLayer:onStart()

    platform_showWaiting(0)
    platform_setVersion_Gdx("version:"..ff.apk_version_name.."."..cc.UserDefault:getInstance():getIntegerForKey("version"))
    hif.init()


    if ff.show_servers == true then
        cclog("测试版本，不连gpg")
        self:start()
        return
    end

    --用gpg登录并且绑定
    print("apk_version:"..ff.apk_version)
    if tonumber(ff.apk_version) <= 1220 then
        self:start()
    else
        local facebook   = cc.UserDefault:getInstance():getStringForKey("facebook","")
        if string.len(facebook) == 32 then
            self:showGPG(false)
            self:start()
        else
            self._back = 0
            self._privacty = 0
            --用gpg登录并且绑定
            self:showGPG(true)
            self._action = schedule(self, self.checkGPGCallBack, 1)       
        end
    end
end

function ProgressLayer.onEnterOrExit(tag)
    cclog("ProgressLayer.onEnterOrExit.........................:"..tag)
    if tag == "enter" then
        --ff.loadingLayer:showTop()
    elseif tag == "enterTransitionFinish" then
        ff.loadingLayer:onStart()
    elseif tag == "exitTransitionStart" then   
        dd.setProgress(90)
    elseif tag == "exit" then    
        dd.setProgress(100)        
        ff.loadingLayer:unscheduleUpdate()        
    elseif tag == "cleanup" then 
        --cc.Director:getInstance():getTextureCache():removeUnusedTextures()     
    end         
end

function ProgressLayer.servers_callback(tag)
    ff.loadingLayer._back = tonumber(tag)
    --cclog("type  ============"..ff.loadingLayer._back)
end

function ProgressLayer.privacy_callback(tag)
    ff.loadingLayer._privacty = tonumber(tag)
    --cclog("type  ============"..ff.loadingLayer._back)
end

function ProgressLayer.auto_login(code)
    cclog("auto_login:"..code)
    ff.loadingLayer._google = ""
    ff.loadingLayer._alias  = ""
    if code ~= "" then
        local tb = Split(code,",fatality,")
        if table.getn(tb) >= 3 then
            local type = tonumber(tb[1])
            local login_alias = tb[2]
            local login_name  = tb[3]
            if type == 4 then
                ff.loadingLayer._google = login_name
                ff.loadingLayer._alias  = login_alias
                cclog("param:"..type..","..login_alias..","..login_name)
            end
        end
    end
    ff.loadingLayer._back = 1
end


function ProgressLayer:showServers()
    platform_registerCallBack(self.servers_callback)
    
    local function isIndex(table_list,index)
        for i=1,table.getn(table_list) do
            if table_list[i] == index then
                return true
            end
        end
        return false
    end

    local server_list_table = {}
    for i=1,hif._lists:rowCount() do
        local type = hif._lists:getLong(i,"type")
        --local result = isIndex(server_list_table  ,type) 
        --if not result then
            local tip = hif._lists:getString(i,"tips")
            platform_showServerList(i,tip)
            --server_list_table[table.getn(server_list_table)+1] = type
        --end
    end
    
end
function ProgressLayer:showGPG(show)
    if show == true then
        platform_showGooglePlayGame(self.auto_login)
    else
        platform_showGooglePlayGame(0)
    end
end



function ProgressLayer:update(dt)
    if self._nowvalue < self._value then
        self._nowvalue = self._nowvalue + 0.5
        --platform_setLoadingBarPercent(self._nowvalue * 10)
    end
end
function ProgressLayer:setVersion()
    --self:getChildByName("version"):setVisible(true)
    --self:getChildByName("version"):setString(getVersion())
end
function ProgressLayer:showVersionDetail()
    local infos = getVersionDetail()
    if table.getn(infos) > 0 then
        for i=1,table.getn(infos) do
            local content = i.."."..infos[i][1]
            local color = infos[i][2]
            platform_showWeihuText(content,color)
        end
    end
end
function runProgressScene(frist)
    --测试
    --cc.UserDefault:getInstance():setIntegerForKey("showbox",2000)
    
    local scene = cc.Scene:create()
    local pLayer
    if ff.platform == 1 then
        pLayer = require "progress/progress_ios"
    else
        pLayer = ProgressLayer
    end

   
    local layer  = pLayer.extend(ccs.GUIReader:getInstance():widgetFromJsonFile("images/loading/loading.json"))
    if layer.initUI then
        layer:initUI()
    end

    scene:addChild(layer) 
    ff.loadingLayer = layer
    platform_setLoadingBarPercent(0)

    if frist == true then
    	if cc.Director:getInstance():getRunningScene() == nil then
    	    cc.Director:getInstance():runWithScene(scene)
    	else
    		cc.Director:getInstance():replaceScene(scene)
    	end
    else
        cc.Director:getInstance():replaceScene(cc.TransitionFadeTR:create(0.3, scene))
    end    
    layer:setProgress(10)
    if frist == true then
        layer:setTips("")	
        ff.sounds[912] = ff.getSounds(912) 
        AudioEngine.playMusic(ff.sounds[912],true)
    else
        layer:setProgress(50)
        layer:setTips(ff.langString("重新登录服务器"))   
    end    
    layer._value = 0
    layer:setVersion()

    layer:scheduleUpdateWithPriorityLua(function (dt)layer:update(dt) end, 0)
    layer:registerScriptHandler(pLayer.onEnterOrExit)
	return layer
end

function showProgressScene()
    local scene = cc.Director:getInstance():getRunningScene()
    local widget = scene:getChildByTag(1)
    if widget ~= nil then
        if dd._cityLayer ~= nil then
            dd._cityLayer:retain()
        end
        widget:removeFromParent(true)
    end
    if dd.mainUI ~= nil then
        dd.mainUI:setVisible(false)
    end

    local pLayer
    if ff.platform == 1 then
        pLayer = require "progress/progress_ios"
    else
        pLayer = ProgressLayer
    end
    local layer  = pLayer.extend(ccs.GUIReader:getInstance():widgetFromJsonFile("images/loading/loading.json"))
    if layer.initUI then
        layer:initUI()
    end

    dd.clearScene()
    ff.loadingLayer = layer
    layer:setProgress(5)
    layer:setTips(ff.langString("重新登录服务器"))   

    layer._value = 0
    layer:setVersion()
    layer:scheduleUpdateWithPriorityLua(function (dt)layer:update(dt) end, 0)


    local function onEnterOrExit(tag)
        cclog("ProgressLayer.onEnterOrExit.........................IOS:"..tag)
        if tag == "enter" then
        elseif tag == "enterTransitionFinish" then
            ff.loadingLayer:onStart()
        elseif tag == "exitTransitionStart" then
            dd.setProgress(90)
        elseif tag == "exit" then    
            dd.setProgress(100)        
            ff.loadingLayer:unscheduleUpdate()        
        elseif tag == "cleanup" then 
        end
    end    
    layer:registerScriptHandler(onEnterOrExit)

    scene:addChild(layer,1,1)
    return layer
end