--[[
世界地图总控制类
单例模式
]]

require "world/worldData"
require "world/worldUILayer"

WorldControl = {}  

WorldControl._worldData = nil	  --世界地图数据层
WorldControl._worldUILayer = nil  --世界地图UI层
WorldControl._init = false
WorldControl._cache = {}


function WorldControl:new(o)  
    o = o or {}  
    setmetatable(o,self)  
    self.__index = self  
    return o  
end

--根据大区图块ID获取图块信息
--[[
@param
number gid = 大区内格子编号，从1开始到1535*1536
]]
function WorldControl:getMapDataByAreaGId(gid)
    --gid已经减掉1了
	local gridData = {}
	local rowIndex = gid%ff.world_maplayerSize.width
	local colIndex = math.floor((gid - rowIndex)/ff.world_maplayerSize.width)
	--大区内属性
	gridData.gid = gid                       	  --大区内编号，从0开始
	gridData.rowIndex = rowIndex             	  --大区内行号，从0开始
	gridData.colIndex = colIndex  				  --大区内列号，从0开始

    local xx = math.ceil((rowIndex + 1)/16)
    local yy = math.ceil((colIndex + 1)/16)
    gridData.areaId = xx + (yy - 1) * 40
	return gridData
end

function WorldControl:init()
    if self._init == false then
        self._init = true

        dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("close"):setTag(1)
        dd.currentLayer._widget:getChildByName("resinfo2"):getChildByName("close"):setTag(2)
        dd.currentLayer._widget:getChildByName("resinfo3"):getChildByName("close"):setTag(3)

        dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("close"):addTouchEventListener(self.onClosePop)
        dd.currentLayer._widget:getChildByName("resinfo2"):getChildByName("close"):addTouchEventListener(self.onClosePop)
        dd.currentLayer._widget:getChildByName("resinfo3"):getChildByName("close"):addTouchEventListener(self.onClosePop)

        dd.currentLayer._widget:getChildByName("map"):getChildByName("map"):getChildByName("info_btn"):addTouchEventListener(self.onShowInfo)
        dd.currentLayer._widget:getChildByName("map"):getChildByName("map"):getChildByName("f2"):addTouchEventListener(self.onClick)
        dd.currentLayer._widget:getChildByName("map"):getChildByName("map"):getChildByName("f3"):addTouchEventListener(self.onClick)
        dd.currentLayer._widget:getChildByName("map"):getChildByName("map"):getChildByName("f4"):addTouchEventListener(self.onClick)

        dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("btn1"):addTouchEventListener(self.onShowUserInfo)
        dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("btn2"):addTouchEventListener(self.onWriteMail)
        dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("btn3"):addTouchEventListener(self.onShowUnion)

        dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("btn1"):addTouchEventListener(self.onShowUserInfo)
        dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("btn2"):addTouchEventListener(self.onWriteMail)
        dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("btn3"):addTouchEventListener(self.onShowUnion)
        dd.currentLayer._widget:getChildByName("resinfo2"):getChildByName("btn1"):addTouchEventListener(self.onShowUserInfo)
        dd.currentLayer._widget:getChildByName("resinfo2"):getChildByName("btn2"):addTouchEventListener(self.onWriteMail)
        dd.currentLayer._widget:getChildByName("resinfo2"):getChildByName("btn3"):addTouchEventListener(self.onShowUnion)

        dd.currentLayer._widget:getChildByName("map"):getChildByName("map"):getChildByName("share"):addTouchEventListener(self.onShare)
        dd.currentLayer._widget:getChildByName("map"):getChildByName("map"):getChildByName("collect"):addTouchEventListener(self.onBook)

        dd.currentLayer._widget:getChildByName("map"):getChildByName("map"):getChildByName("f2"):getChildByName("icon"):setPositionY(28)
        dd.currentLayer._widget:getChildByName("map"):getChildByName("map"):getChildByName("f2"):getChildByName("label"):setPositionY(28)
        dd.currentLayer._widget:getChildByName("map"):getChildByName("map"):getChildByName("f3"):getChildByName("icon"):setPositionY(28)
        dd.currentLayer._widget:getChildByName("map"):getChildByName("map"):getChildByName("f3"):getChildByName("label"):setPositionY(28)
        dd.currentLayer._widget:getChildByName("map"):getChildByName("map"):getChildByName("f4"):getChildByName("icon"):setPositionY(28)
        dd.currentLayer._widget:getChildByName("map"):getChildByName("map"):getChildByName("f4"):getChildByName("label"):setPositionY(28)


        dd.currentLayer._widget:getChildByName("map"):addTouchEventListener(self.onCloseMap)

    end
end
function WorldControl.onCloseMap(sender,eventType)
    if eventType==ccui.TouchEventType.ended then     
        WorldControl:getInstance():closeAll()
    end
end
--按钮
function WorldControl.onShowUserInfo(sender,eventType)
    if eventType==ccui.TouchEventType.ended then   
        local widget = WorldControl:getInstance()
        if widget._cache['uid'] == dd.uid then
            pop.onShowPopDialog("powerInfo")
        else
            pop.onShowPopDialog("power2",dd.currentLayer._widget:getChildByName("pop3"),widget._cache['uid'])    
        end
        widget:closeAll()
    end
end
function WorldControl.onWriteMail(sender,eventType)
    if eventType==ccui.TouchEventType.ended then 
        local widget = WorldControl:getInstance()
        if widget._cache['uid'] == dd.uid then
            dd.currentLayer._widget:onShowTips("\t"..ff.langString("不能给自己写信"))
        else
            pop.onShowPopDialog("mailWrite",true,widget._cache['uid'],widget._cache['name'],ff.langString("打个招呼"))
        end
        widget:closeAll()
    end
end
function WorldControl.onShowUnion(sender,eventType)
    if eventType==ccui.TouchEventType.ended then   
        local widget = WorldControl:getInstance()
        if widget._cache['uid'] == dd.uid then
            if dd.myUnion:getLong(1,"unionid") <= 0 then
                pop.onShowPopDialog("noUnion",1)
            else
                pop.onShowPopDialog("union",1)
            end
        else
            if dd.myUnion:getLong(1,"unionid") > 0 then
                if dd.myUnion:getString(1,"unionflag") == widget._cache['short'] then
                    pop.onShowPopDialog("union",1)
                    widget:closeAll()
                    return
                else
                    if widget._cache['unionid'] > 0 then
                        pop.onShowPopDialog("unionInfo",dd.currentLayer._widget,widget._cache['short'])
                        widget:closeAll()
                        return
                    end
                end
                if (widget.hasUnion) then
                    local unionofficial=dd.myUnion:getLong(1,"unionofficial")
                    if(unionofficial>=3) then
                        cmd.iviteToGroup(widget._cache['name'])
                    else
                        dd.currentLayer._widget:onShowTips("\t"..ff.langString("您没有邀请的权限"))
                    end
                end
            else
                if widget._cache['unionid'] > 0 then
                    --cmd.joinToGroup(widget._cache['unionid'])
                    local function joinGroupBack(ret)
                        if ret[1] == 0 then
                            if ret[2]['action'] == 1 then
                                flag = true
                                if ret[2]['join_suc'] == 0 then
                                    dd.currentLayer._widget:onShowTips("\t"..ff.langString("申请已发出，请等待"),true)
                                elseif ret[2]['join_suc'] == 1 then
                                    dd.currentLayer._widget:onShowTips("\t"..ff.langString("申请已被接受，成功加入联盟"),true)
                                    pop.onClosePopDialog("unionInfo")
                                    performWithDelay(dd.currentLayer._widget,function() pop.onShowPopDialog("union",1) end,0.2)
                                end
                            end
                        else
                            if dd.currentLayer ~= nil and dd.currentLayer._widget ~= nil then
                                dd.currentLayer._widget:onShowTips("\t"..dd.errorCode(ret[1]))
                            end 
                        end
                        ff.net:removeEvent(5,dd.Tjf_NewAllyApply)
                    end
                    ff.net:registerEvent(5,dd.Tjf_NewAllyApply,function (message) joinGroupBack(message) end)
                    local message = ff.net:createMessage()
                    message:addCmd(0x122)
                    message:addLong(dd.uid)
                    message:addLong(1) 
                    message:addLong(widget._cache['unionid'])
                    ff.net:rpc(message,false)
                    dd.myUnion:setString(1,"unionflag",widget._cache['short'])
                end
            end
        end
        widget:closeAll()
    end
end
function WorldControl:closeAll()
    dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("detail"):getChildByName("info"):getChildByName("pg"):getChildByName("progress"):stopAllActions()
    local view = dd.currentLayer._widget:getChildByName("resinfo1")
    if view:isVisible() == true then
        local function unShow()
            view:setVisible(false)
        end    
        view:runAction(cc.Sequence:create(cc.MoveTo:create(0.5, cc.p(0,-view:getContentSize().height)),cc.CallFunc:create(unShow)))
    end
    view = dd.currentLayer._widget:getChildByName("resinfo2")
    if view:isVisible() == true then
        local function unShow()
            view:setVisible(false)
        end    
        view:runAction(cc.Sequence:create(cc.MoveTo:create(0.5, cc.p(0,-view:getContentSize().height)),cc.CallFunc:create(unShow)))
    end
    view = dd.currentLayer._widget:getChildByName("resinfo3")
    if view:isVisible() == true then
        local function unShow()
            view:setVisible(false)
        end    
        view:runAction(cc.Sequence:create(cc.MoveTo:create(0.5, cc.p(0,-view:getContentSize().height)),cc.CallFunc:create(unShow)))
    end
    self:closePannel()
end
function WorldControl.onClosePop(sender,eventType)
    if eventType==ccui.TouchEventType.ended then     
        local tag = sender:getTag()
        local view = nil
        if tag == 1 then
            view = dd.currentLayer._widget:getChildByName("resinfo1")
        elseif tag == 2 then
            view = dd.currentLayer._widget:getChildByName("resinfo2")
        elseif tag == 3 then
            view = dd.currentLayer._widget:getChildByName("resinfo3")
        end
        dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("detail"):getChildByName("info"):getChildByName("pg"):getChildByName("progress"):stopAllActions()
        if view ~= nil then
            view:setVisible(false)

        end            
    end
end
function WorldControl.onShowInfo(sender,eventType)
    if eventType==ccui.TouchEventType.ended then     
        local tag = sender:getTag()

        WorldControl:getInstance():onShowDetail(tag)
    end
end
function WorldControl:onShowDetail(gid)
    cclog("onShowDetail")
    dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("detail"):getChildByName("info"):getChildByName("pg"):getChildByName("progress"):stopAllActions()
    local view = nil
    --cclog("gid:"..gid..","..self._cache['cid']..","..self._cache['troopid']..self._cache['res'])
    if self._cache['cid'] == gid then
        local map = dd.currentLayer._widget:getChildByName("map")
        map:setTag(3)
        map:getChildByName("map"):setTag(gid)

        if self._cache['troopid'] > 0 then
            view = dd.currentLayer._widget:getChildByName("resinfo3")
            if self._cache['state'] == 2 then
                view:getChildByName("info"):getChildByName("tips"):setString(ff.langString("撤回")..":"..self:getCidStr(self._cache['to']))
            else
                cclog("self._cache['state']:"..self._cache['state'])
                if self._cache['state'] == 4 then
                    view:getChildByName("info"):getChildByName("tips"):setString(ff.langString("驻扎")..":"..self:getCidStr(self._cache['to']))
                elseif self._cache['state'] == 5 then
                    view:getChildByName("info"):getChildByName("tips"):setString(ff.langString("增援")..":"..self:getCidStr(self._cache['to']))
                elseif self._cache['state'] == 8 then
                    view:getChildByName("info"):getChildByName("tips"):setString(ff.langString("地下城堡主要考验的是部队的生命属性。地下城堡中探险不会使你的部队死亡，而是直接撤退回城市中。行动力不足后，部队将自动撤回。"))
                else
                    view:getChildByName("info"):getChildByName("tips"):setString(ff.langString("行军")..":"..self:getCidStr(self._cache['to']))
                end
            end
            view:requestDoLayout()
        else
            if self._cache['res'] == 0 then
                --空地

                view = dd.currentLayer._widget:getChildByName("resinfo3")
                if self._cache['uid'] == 998 then

                    view:getChildByName("face"):getChildByName("face"):loadTexture("images/icon/que/Exploring.png",0)
                    view:getChildByName("info"):getChildByName("tips"):setString(ff.langString("地下城堡"))
                    view:getChildByName("title"):setString(ff.langString("可以派兵驻扎"))
                else
                    local image = self._worldUILayer._curMapLayer:getCidImage(self._worldUILayer:getCid(self._cache['cid']-1))
                    if image == "" then
                        view:getChildByName("face"):getChildByName("face"):loadTexture("world/701.png",1)
                    else
                        view:getChildByName("face"):getChildByName("face"):loadTexture(image,1)
                    end
                    view:getChildByName("info"):getChildByName("tips"):setString(ff.langString("空地"))
                    view:getChildByName("title"):setString(ff.langString("可以派兵驻扎"))
                end
            elseif self._cache['res'] == 9 then
                view = dd.currentLayer._widget:getChildByName("resinfo3")
                
                local image = self._worldUILayer._curMapLayer:getCidImage(self._worldUILayer:getCid(self._cache['cid']))
                
                if image == "" then
                    view:getChildByName("face"):getChildByName("face"):loadTexture("world/701.png",1)
                else
                    view:getChildByName("face"):getChildByName("face"):loadTexture(image,1)
                end
                
                view:getChildByName("info"):getChildByName("tips"):setString(ff.langString("据点"))
                view:getChildByName("title"):setString(ff.langString("可以派兵驻扎"))
            elseif self._cache['res'] == 10 then
                view = dd.currentLayer._widget:getChildByName("resinfo3")
                
                
                view:getChildByName("face"):getChildByName("face"):loadTexture("world/702.png",1)
               
                
                view:getChildByName("info"):getChildByName("tips"):setString(ff.langString("哨塔"))
                view:getChildByName("title"):setString(ff.langString("可以派兵驻扎"))
            elseif self._cache['res'] == 11 then
                view = dd.currentLayer._widget:getChildByName("resinfo3")
                
                view:getChildByName("face"):getChildByName("face"):loadTexture("images/active/861.png",0)
                
                
                view:getChildByName("info"):getChildByName("tips"):setString(ff.langString("核心"))
                view:getChildByName("title"):setString(ff.langString("可以派兵驻扎"))
            elseif self._cache['res'] == 7 or self._cache['res'] == 8 then
                view = dd.currentLayer._widget:getChildByName("resinfo2")
                if self._cache['uid'] == dd.uid then
                    view:getChildByName("detail"):getChildByName("face"):getChildByName("face"):loadTexture(dd.getPlayerIcon(dd.myinfo:getLong(1,"face")),0)
                    view:getChildByName("detail"):getChildByName("info"):getChildByName("name"):setString(dd.myinfo:getString(1,"name"))
                    view:getChildByName("detail"):getChildByName("level"):getChildByName("level"):setString(" LV."..dd.myinfo:getLong(1,"level"))
                    if dd.myUnion:getLong(1,"unionid") > 0 then
                        view:getChildByName("detail"):getChildByName("info"):getChildByName("union"):setString("["..dd.myUnion:getString(1,"unionflag").."]"..dd.myUnion:getString(1,"unionname"))
                    else
                        view:getChildByName("detail"):getChildByName("info"):getChildByName("union"):setString(ff.langString("暂无"))
                    end                
                    view:getChildByName("detail"):getChildByName("info"):getChildByName("power"):setString(ff.getFormatMoney(dd.myinfo:getLong(1,"power")))
                else
                    view:getChildByName("detail"):getChildByName("face"):getChildByName("face"):loadTexture(dd.getPlayerIcon(self._cache['face']),0)
                    view:getChildByName("detail"):getChildByName("info"):getChildByName("name"):setString(self._cache['name'])
                    view:getChildByName("detail"):getChildByName("level"):getChildByName("level"):setString(" LV."..self._cache['level'])

                    if self._cache['unionid'] > 0 then
                        view:getChildByName("detail"):getChildByName("info"):getChildByName("union"):setString("["..self._cache['short'].."]"..self._cache['uname'])
                    else
                        view:getChildByName("detail"):getChildByName("info"):getChildByName("union"):setString(ff.langString("暂无"))
                    end

                     --wh
                    view:getChildByName("btn3"):setVisible(true)
                    if dd.myUnion:getLong(1,"unionid") > 0 then
                        if self._cache['unionid'] > 0 then
                            view:getChildByName("btn3"):getChildByName("chu_label"):setString(ff.langString("联盟信息"))
                        else
                            view:getChildByName("btn3"):getChildByName("chu_label"):setString(ff.langString("邀请"))
                            local hasUnion = false
                            if(dd.myUnion) then
                                local unionid=dd.myUnion:getLong(1,"unionid")
                                if(unionid~=0) then
                                    hasUnion=true
                                end
                            end
                            self.hasUnion=hasUnion
                        end
                    else
                        if self._cache['unionid'] > 0 then
                            view:getChildByName("btn3"):getChildByName("chu_label"):setString(ff.langString("加入"))
                        else
                            view:getChildByName("btn3"):setVisible(false)
                        end
                    end 

                    cclog("myUnion: "..dd.myUnion:getString(1,"unionflag"))
                    cclog("self._cache['short']:"..self._cache['short'])

                    view:getChildByName("detail"):getChildByName("info"):getChildByName("power"):setString(ff.getFormatMoney(self._cache['power']))
                end                
            elseif self._cache['res'] >= 1 and self._cache['res'] <= 6 or self._cache['res'] == 13 then     
                --资源       
                if self._cache['uid'] == dd.uid then
                    local function setProgress()
                        local gets = self._cache['gets'] + math.ceil((dd.getServerTime() - self._cache['tick']) * self._cache['speed']/3600)
                        cclog("gets:"..gets..",cur:"..self._cache['cur']..",gets2:"..self._cache['gets'])
                        local widget = dd.currentLayer._widget:getChildByName("map")

                        local progress = 0
                        cclog("cary:"..self._cache['carry']..",cur:"..self._cache['cur'])
                        if self._cache['cary'] > self._cache['cur'] then
                            progress = math.floor(100 * gets/self._cache['cur'])      
                            view:getChildByName("detail"):getChildByName("count"):setString(gets.."/"..self._cache['cur'])          
                        else    
                            progress = math.floor(100 * gets/self._cache['carry'])                
                            view:getChildByName("detail"):getChildByName("count"):setString(gets.."/"..self._cache['carry'])          
                        end
                        if progress > 100 then
                            progress = 100
                        end
                        local width = (412 * progress)/100
                        if width > 412 then
                            width = 412
                        end
                        cclog("width:"..width..",pro:"..progress)
                        view:getChildByName("detail"):getChildByName("info"):getChildByName("pg"):getChildByName("progress"):setContentSize(cc.size(width,36))    
                        view:getChildByName("detail"):getChildByName("info"):getChildByName("pg"):getChildByName("tag"):getChildByName("label"):setString(progress.."%")
                        view:getChildByName("detail"):getChildByName("info"):getChildByName("pg"):getChildByName("tag"):setPositionX(width)                    
                    end
                    view = dd.currentLayer._widget:getChildByName("resinfo1")
                    local image  = dd.cfgWorldRes[math.floor(self._cache['avatar']/10000)]['image']
                    local level  = dd.cfgWorldRes[math.floor(self._cache['avatar']/10000)]['level']
                    view:getChildByName("detail"):getChildByName("level"):getChildByName("label"):setString("LV."..level)

                    view:getChildByName("detail"):getChildByName("face"):getChildByName("face"):loadTexture("world/"..image..".png",1)
                    view:getChildByName("detail"):getChildByName("info"):getChildByName("name"):setString(dd.myinfo:getString(1,"name").." LV."..dd.myinfo:getLong(1,"level"))
                    if dd.myUnion:getLong(1,"unionid") > 0 then
                        view:getChildByName("detail"):getChildByName("info"):getChildByName("union"):setString("["..dd.myUnion:getString(1,"unionflag").."]"..dd.myUnion:getString(1,"unionname"))
                    else
                        view:getChildByName("detail"):getChildByName("info"):getChildByName("union"):setString(ff.langString("暂无"))
                    end

                    --wh
                    view:getChildByName("btn3"):setVisible(true)
                    if dd.myUnion:getLong(1,"unionid") > 0 then
                        if self._cache['unionid'] > 0 then
                            view:getChildByName("btn3"):getChildByName("chu_label"):setString(ff.langString("联盟信息"))
                        else
                            view:getChildByName("btn3"):getChildByName("chu_label"):setString(ff.langString("邀请"))
                            local hasUnion = false
                            if(dd.myUnion) then
                                local unionid=dd.myUnion:getLong(1,"unionid")
                                if(unionid~=0) then
                                    hasUnion=true
                                end
                            end
                            self.hasUnion=hasUnion
                        end
                    else
                        if self._cache['unionid'] > 0 then
                            view:getChildByName("btn3"):getChildByName("chu_label"):setString(ff.langString("加入"))
                        else
                            view:getChildByName("btn3"):setVisible(false)
                        end
                    end       

                    view:getChildByName("detail"):getChildByName("info"):getChildByName("power"):setString(ff.getFormatMoney(dd.myinfo:getLong(1,"power")))
                    view:getChildByName("detail"):getChildByName("info"):getChildByName("tag"):getChildByName("res"):loadTexture(dd.getPathFromUI("public/ic0"..self._cache['res']..".png",1))

                    setProgress()
                    schedule(view:getChildByName("detail"):getChildByName("info"):getChildByName("pg"):getChildByName("progress"), setProgress, 1)
                else
                    local image  = dd.cfgWorldRes[math.floor(self._cache['avatar']/10000)]['image']
                    local level  = dd.cfgWorldRes[math.floor(self._cache['avatar']/10000)]['level']

                    if self._cache['uid'] == 0 then
                        view = dd.currentLayer._widget:getChildByName("resinfo3")
                        view:getChildByName("title"):setString(ff.langString("派遣军队->采集资源"))
                        view:getChildByName("face"):getChildByName("face"):loadTexture("world/"..image..".png",1)
                        view:getChildByName("info"):getChildByName("tips"):setString(ff.langString(dd.cfgWorldRes[math.floor(self._cache['avatar']/10000)]['name']).." LV."..level)
                    else
                        view = dd.currentLayer._widget:getChildByName("resinfo2")
                        view:getChildByName("detail"):getChildByName("face"):getChildByName("face"):loadTexture("world/"..image..".png",1)
                        view = dd.currentLayer._widget:getChildByName("resinfo2")
                        view:getChildByName("detail"):getChildByName("level"):getChildByName("level"):setString("LV."..level)

                        --wh
                        view:getChildByName("btn3"):setVisible(true)
                        if dd.myUnion:getLong(1,"unionid") > 0 then
                            if self._cache['unionid'] > 0 then
                                view:getChildByName("btn3"):getChildByName("chu_label"):setString(ff.langString("联盟信息"))
                            else
                                view:getChildByName("btn3"):getChildByName("chu_label"):setString(ff.langString("邀请"))
                                local hasUnion = false
                                if(dd.myUnion) then
                                    local unionid=dd.myUnion:getLong(1,"unionid")
                                    if(unionid~=0) then
                                        hasUnion=true
                                    end
                                end
                                self.hasUnion=hasUnion
                            end
                        else
                            if self._cache['unionid'] > 0 then
                                view:getChildByName("btn3"):getChildByName("chu_label"):setString(ff.langString("加入"))
                            else
                                view:getChildByName("btn3"):setVisible(false)
                            end
                        end 
                    end        
                end
            end    
        end
    end
    dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("detail"):getChildByName("info"):requestDoLayout()
    if view ~= nil then
        if view:getActionByTag(99) == nil then
            if view:isVisible() == false or view:getPositionY() ~= 214 then
                view:setVisible(true)
                view:setPositionY(-view:getContentSize().height)
                local action = view:runAction(cc.MoveTo:create(0.5, cc.p(0,214)))
                action:setTag(99)
            else
                view:setPositionY(214)
                view:setVisible(true)
            end
        end
    end    
end
function WorldControl.onClick(sender,eventType)
    if eventType==ccui.TouchEventType.ended then     
        local tag = sender:getTag()
        cclog("tag:"..tag)
        if tag <= 0 then
            return
        end
        local function goCity(tid)
            local message = ff.net:createMessage()
            message:addCmd(0x00000137)
            message:addLong(dd.uid)
            message:addLong(tid)
            message:addLong(WorldControl:getInstance()._cache['cid'])
            ff.net:rpc(message,false)
            WorldControl:getInstance():closeAll()
        end
        local function addSpeed(tid)
            local message = ff.net:createMessage()
            message:addCmd(0x0000012f)
            message:addLong(dd.uid)
            if dd.getToolsCount(tid) > 0 then
                message:addLong(1)
            else
                message:addLong(2)
            end
            message:addLong(WorldControl:getInstance()._cache['troopid'])
            message:addLong(tid)
            ff.net:rpc(message,false)
            WorldControl:getInstance():closeAll()
        end

        local function click()
            AudioEngine.playEffect(ff.sounds[101])
            if tag == 1 or tag == 19 then
                --cclog("=22222222222222222222222222222222==============="..WorldControl:getInstance()._cache["level"])
                pop.onShowPopDialog("chuzhen",WorldControl:getInstance()._cache['type'],WorldControl:getInstance()._cache)
                WorldControl:getInstance():closeAll()
            elseif tag == 2 or tag == 11 then
                pop.onShowPopDialog("chuzhen",WorldControl:getInstance()._cache['type'],WorldControl:getInstance()._cache)
                WorldControl:getInstance():closeAll()
            elseif tag == 17 then
                pop.onShowPopDialog("powerInfo",1)
                WorldControl:getInstance():closeAll()
            elseif tag == 20 then
                pop.onShowPopDialog("changeSkin",1)
                WorldControl:getInstance():closeAll()
            elseif tag == 18 then
                local params = {}
                params.touid = WorldControl:getInstance()._cache['uid']
                params.totem = 0
                params.level= WorldControl:getInstance()._cache['level']
                params.name = WorldControl:getInstance()._cache['name']
                params.cid = WorldControl:getInstance()._cache['cid']
                pop.onShowPopDialog("transport",params)    
                WorldControl:getInstance():closeAll()            
            elseif tag == 3 then
                local function realsend()
                    local message = ff.net:createMessage()
                    message:addCmd(0x0000010f)
                    message:addLong(dd.uid)
                    message:addLong(WorldControl:getInstance()._cache['cid'])
                    message:addLong(3)
                    message:addLong(0)
                    message:addLong(0)
                    for i=1,40 do
                        message:addLong(0)
                    end
                    ff.net:rpc(message,true)
                    WorldControl:getInstance():closeAll()
                end
                local function onSend()
                    dd.currentLayer._widget:showPeek(WorldControl:getInstance()._cache,realsend)
                end
                local function onSure()
                    local message = ff.net:createMessage()
                    message:addCmd(0x00000126)
                    message:addLong(dd.uid)
                    if dd.myinfo:getLong(1,"state") == 1 then
                        message:addLong(101)
                    else    
                        message:addLong(102)
                    end 
                    ff.net:rpc(message,true)      
                    onSend()              
                end
                --侦查
                if WorldControl:getInstance()._cache['protect'] == 1 and WorldControl:getInstance()._cache['res'] == 7 then
                    dd.currentLayer._widget:onShowTips("\t"..ff.langString("对方处于保护中，无法被侦查。"))
                    WorldControl:getInstance():closePannel()
                    return
                end        
                if dd.myinfo:getLong(1,"state") == 1 then
                    dd.currentLayer._widget:onShowSure(ff.langString("解除保护"),ff.langString("您目前处于保护状态,无法出兵.\n是否解除保护?"),onSure,true,2)
                    WorldControl:getInstance():closePannel()
                    return
                else                
                    for i=1,dd.myplus:rowCount() do
                        if dd.myplus:getLong(i,"id") == 531 then
                            dd.currentLayer._widget:onShowSure(ff.langString("解除保护"),ff.langString("您目前处于保护状态,无法出兵.\n是否解除保护?"),onSure,true,2)
                            WorldControl:getInstance():closePannel()
                            return
                        end
                    end
                end                 
                onSend()
                WorldControl:getInstance():closePannel()
            elseif tag == 4 then
                if dd.myUnion:getLong(1,"unionid") <= 0 then
                    dd.currentLayer._widget:onShowTips("\t"..ff.langString("必须先加入联盟，才能发起联合攻击。"))                
                else         

                    local build = false
                    for i=1,dd.mybuilds:rowCount() do
                        if dd.mybuilds:getLong(i,"bid") == 15 then
                            if dd.myinfo:getLong(1,"level") < dd.cfgPos[dd.mybuilds:getLong(i,"pos")]['lv'] then      
                                dd.currentLayer._widget:onShowTips("\t"..ff.langString("司令部等级达{0}级开放",{dd.cfgPos[dd.mybuilds:getLong(i,"pos")]['lv']}))
                                build = true
                            else
                                local maxarmys = dd.cfgBuildingUp[15][dd.mybuilds:getLong(i,"level")]['value1']
                                WorldControl:getInstance()._cache['maxarmys'] = maxarmys
                                if WorldControl:getInstance()._cache['type'] ~= 23 then
                                    WorldControl:getInstance()._cache['type'] = 5
                                end
                                pop.onShowPopDialog("chuzhen",WorldControl:getInstance()._cache['type'],WorldControl:getInstance()._cache)
                                WorldControl:getInstance():closeAll()
                                build = true
                            end
                            
                            break
                        end
                    end
                    if build == false then
                        dd.currentLayer._widget:onShowTips("\t"..ff.langString("必须先建造 战争会所"))
                    end
                    
                end
            elseif tag == 5 then
                --迁城
                local maxTroops = 0
                for i=1,dd.myTroops:rowCount() do
                    if dd.myTroops:getLong(i,"type") ~= 13 then
                        maxTroops = maxTroops + 1 
                    end
                end
                if maxTroops > 0 then
                    dd.currentLayer._widget:onShowTips("\t"..ff.langString("有部队在城市外，无法迁徙"))               
                else
                    dd.currentLayer._widget:onShowMulTools(18,0,goCity)
                end    
            elseif tag == 24 then
                --迁城
                local maxTroops = 0
                for i=1,dd.myTroops:rowCount() do
                    if dd.myTroops:getLong(i,"type") ~= 13 then
                        maxTroops = maxTroops + 1 
                    end
                end
                if maxTroops > 0 then
                    dd.currentLayer._widget:onShowTips("\t"..ff.langString("有部队在城市外，无法迁徙"))               
                else
                    dd.currentLayer._widget:onShowMulTools(20,0,goCity)
                end    
            elseif tag == 6 then
                --回城
                for i=1,dd.myTroops:rowCount() do
                    if dd.myTroops:getLong(i,"to") == WorldControl:getInstance()._cache['cid'] then
                        local function callback()
                            local message = ff.net:createMessage()
                            message:addCmd(0x00000110)
                            message:addLong(dd.uid)
                            message:addLong(dd.myTroops:getLong(i,"id"))
                            ff.net:rpc(message,false)
                            AudioEngine.playEffect(ff.sounds[412])
                        end
                        if dd.myTroops:getLong(i,"state") <= 2 or dd.myTroops:getLong(i,"state") == 7 then
                            dd.currentLayer._widget:onShowSureUseTools(ff.langString("部队召回"),1018,1,callback)                            
                        else
                            callback()
                        end
                        break
                    end
                end
                WorldControl:getInstance():closePannel()
            elseif tag == 16 then
                --回城
                for i=1,dd.myTroops:rowCount() do
                    if dd.myTroops:getLong(i,"id") == WorldControl:getInstance()._cache['troopid'] then
                        local function callback()
                            local message = ff.net:createMessage()
                            message:addCmd(0x00000110)
                            message:addLong(dd.uid)
                            message:addLong(dd.myTroops:getLong(i,"id"))
                            ff.net:rpc(message,false)
                            AudioEngine.playEffect(ff.sounds[412])
                        end
                        if dd.myTroops:getLong(i,"state") <= 2 or dd.myTroops:getLong(i,"state") == 7 then
                            dd.currentLayer._widget:onShowSureUseTools(ff.langString("部队召回"),1018,1,callback)                            
                        else
                            callback()
                        end
                        break
                    end
                end
                WorldControl:getInstance():closePannel()
            elseif tag == 7 then
                --进入城市
                dd.yunClose(1)
                WorldControl:getInstance():closePannel()
            elseif tag == 21 then
                --进入城市
                pop.onShowPopDialog("StrongholdInfo",WorldControl:getInstance()._cache['id']) 
            elseif tag == 23 then
                --进入城市
                local have = false
                for i=1,dd.mybuilds:rowCount() do
                    if dd.mybuilds:getLong(i,"bid") == 15 then
                        onShowBuildsList(dd.currentLayer._widget,dd.mybuilds:getLong(i,"pos")) 
                        have = true
                    end
                end
                if have == false then
                    dd.currentLayer._widget:onShowTips("\t"..ff.langString("请先建造{0}",{ff.langString(dd.cfgBuilding[15]['name'])}),true)
                end
            elseif tag == 22 then
                --进入城市
                pop.onShowPopDialog("StrongholdInfoShow") 
            elseif tag == 28 then
                --进入城市
                pop.onShowPopDialog("StrongholdInfoEEX",WorldControl:getInstance()._cache['uid'],WorldControl:getInstance()._cache['cid'] )  
            elseif tag == 27 then
                if WorldControl:getInstance()._worldData._Tower[WorldControl:getInstance()._cache['cid']] ~= nil then
                    pop.onShowPopDialog("StrongTower",WorldControl:getInstance()._worldData._Tower[WorldControl:getInstance()._cache['cid']],WorldControl:getInstance()._cache['id'])   
                end
            elseif tag == 25 then
                    dd.currentLayer._widget:onShowTools()
            elseif tag == 8 then
                --增援
                WorldControl:getInstance()._cache['type'] = 4
                pop.onShowPopDialog("chuzhen",WorldControl:getInstance()._cache['type'],WorldControl:getInstance()._cache)
                WorldControl:getInstance():closeAll()
            elseif  tag == 26 then
                pop.onShowPopDialog("StrongholdInfoShow",2)
            elseif  tag == 29 then
                pop.onShowPopDialog("StrongholdInfoShow",4) 
                
            elseif tag == 9 then
                --查看部队
                --self._cache['troopid'] 
                if WorldControl:getInstance()._cache['troopid'] > 0 then
                    for j=1,dd.myTroops:rowCount() do
                        if dd.myTroops:getLong(j,"id") == WorldControl:getInstance()._cache['troopid'] then
                            if dd.myTroops:getLong(j,"type") == 11 then
                                pop.onShowPopDialog("transportInfo",WorldControl:getInstance()._cache['troopid'])
                            else
                                pop.onShowPopDialog("armyInfo",WorldControl:getInstance()._cache['troopid'])
                            end
                            break
                        end
                    end
                else
                    for j=1,dd.myTroops:rowCount() do
                        if dd.myTroops:getLong(j,"to") == WorldControl:getInstance()._cache['cid'] then
                            pop.onShowPopDialog("armyInfo",dd.myTroops:getLong(j,"id"))
                            break
                        end
                    end
                end
                WorldControl:getInstance():closePannel()
            elseif tag == 10 then
                dd.currentLayer._widget:onShowMulTools(29,0,addSpeed)
                WorldControl:getInstance():closeAll()
            end
        end
        local action  = cc.EaseExponentialOut:create(cc.ScaleTo:create(0.1, 1.05,1))
        local action2 = cc.EaseExponentialOut:create(cc.ScaleTo:create(0.05, 1,1))
        sender:runAction(cc.Sequence:create(action,action2,cc.CallFunc:create(click)))
    end
end
function WorldControl:closePannel()
    dd.currentLayer._widget:getChildByName("map"):setVisible(false)
    dd.currentLayer._widget:getChildByName("map"):getChildByName("map"):getChildByName("progress"):stopAllActions()
    dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("detail"):getChildByName("info"):getChildByName("pg"):getChildByName("progress"):stopAllActions()
end
function WorldControl:getCidStr(gid)
    local x = (gid-1)%ff.world_maplayerSize.width
    local y = math.floor((gid-1 - x)/ff.world_maplayerSize.width)
    return "X:"..(x+1)..",Y:"..(y+1)
end
--显示NPC
function WorldControl:showNpcPannel(data)
    local type = data.type
    local id   = data.uid
    cclog("选中NPC："..id..","..type)    
    --显示攻击NPC面板
    local params = {}
    params.cid = data.cid
    params.uid = dd.uid
    params.unionid = data.unionid
    params.short = data.short
    params.name = data.name
    params.level= data.level
    params.npcid= data.uid
    if type == 21 then
        pop.onShowPopDialog("mapNpc",params)
    end
    --pop.onShowPopDialog("chuzhen",type,params)
end
function WorldControl:showNpcPannelEx(data)
    AudioEngine.playEffect(ff.sounds[101])
        
    self._cache['cid']   = data.cid
    self._cache['uid']   = data.uid
    self._cache['type']  = data.type
    self._cache['res_type']  = data.type
    self._cache['res_id']    = math.floor(data.avatar/10000)

    self._cache['avatar']= data.avatar
    self._cache['level'] = data.level
    self._cache['cur']   = 0
    self._cache['cary']  = 0
    self._cache['unionid']  = data.unionid
    self._cache['uname']    = ""
    self._cache['short']    = data.short
    self._cache['ulevel']   = 0
    self._cache['name']     = data.name
    self._cache['count']    = 0
    self._cache['res']      = 0
    self._cache['friend']   = 0
    self._cache['protect']  = 0
    self._cache['troopid']  = 0
    self._cache.npcid = data.uid


    local map = dd.currentLayer._widget:getChildByName("map")
    if map:isVisible() == true then
        self:closePannel()
        return
    end
    map:getChildByName("map"):getChildByName("mask"):stopAllActions()
    --local action = cc.Blink:create(3, 3)
    --map:getChildByName("map"):getChildByName("mask"):runAction(cc.RepeatForever:create(cc.Sequence:create(action,action:reverse())))


    local widget = map:getChildByName("map")
    map:setTag(4)
    map:getChildByName("map"):setTag(data.uid)
    --显示按钮
    widget:getChildByName("f2"):setVisible(false)
    widget:getChildByName("f3"):setVisible(false)
    widget:getChildByName("f4"):setVisible(false)

    widget:getChildByName("f2"):setTag(0)
    widget:getChildByName("f3"):setTag(0)
    widget:getChildByName("f4"):setTag(0)

    widget:getChildByName("info"):getChildByName("name"):setVisible(true)
    widget:getChildByName("info"):getChildByName("city"):setVisible(true)
    widget:getChildByName("info"):getChildByName("name"):setString(ff.langString("地下城堡"))
    widget:getChildByName("info"):getChildByName("icon"):setVisible(false)
    widget:getChildByName("info"):getChildByName("city"):setString(self:getCidStr(data.cid))
    
    widget:getChildByName("f3"):setTag(19)
    widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("探险"))

    widget:getChildByName("info_btn"):setTag(data.cid)
    widget:getChildByName("info"):setContentSize(cc.size(widget:getChildByName("info"):getChildByName("name"):getContentSize().width +40,60))
    widget:getChildByName("info"):getChildByName("name"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
    widget:getChildByName("info"):getChildByName("city"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)

   
    dd.currentLayer._widget:getChildByName("resinfo3"):getChildByName("face"):getChildByName("face"):loadTexture("images/icon/que/Exploring.png",0)
    local function startAction()
        widget:getChildByName("share"):setVisible(false)
        widget:getChildByName("collect"):setVisible(false)


        widget:getChildByName("info_btn"):setPositionX(145)
        widget:getChildByName("info_btn"):setPositionY(145)
        widget:getChildByName("info_btn"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(30,65))))

        widget:getChildByName("back2"):setScale(0.1)
        widget:getChildByName("back2"):runAction(cc.ScaleTo:create(0.5, 1))

        widget:getChildByName("info"):setScaleX(0.1)
        widget:getChildByName("info"):runAction(cc.EaseExponentialOut:create(cc.ScaleTo:create(0.5, 1)))
        dd.currentLayer._widget:getChildByName("map"):setVisible(true)

        local tick = 0
        for i=1,3 do
            local j = 5 - i
            local tag = widget:getChildByName("f"..j):getTag()
            if tag > 0 then
                local icon = tag
                if icon == 16 then
                    icon = 6
                end
                widget:getChildByName("f"..j):getChildByName("icon"):loadTexture("images/icon/world/ic"..icon.."_map.png")
                widget:getChildByName("f"..j):setVisible(true)
                widget:getChildByName("f"..j):setPositionX(90)
                --widget:getChildByName("f"..j):getChildByName("label"):setPositionY(18)
                widget:getChildByName("f"..j):requestDoLayout()

                local fx = 230
                if j == 3 then
                    fx = 250
                end
                if tick > 0 then
                    widget:getChildByName("f"..j):runAction(cc.Sequence:create(cc.DelayTime:create(tick),cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY())))))
                else
                    widget:getChildByName("f"..j):runAction(cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY()))))
                end
                --tick = tick + 0.1           
            end    
        end
    end       


    widget:stopAllActions()
    widget:setPosition(cc.p(cc.p(ff.width/2,(ff.height - 221)/2 + 221)))
    performWithDelay(widget, startAction, 0.1)
end
function WorldControl:showPannel(data,point,showimg)   
    self._worldUILayer._flowid = 0
    AudioEngine.playEffect(ff.sounds[101])

    cclog("显示大圆钮")
    local map = dd.currentLayer._widget:getChildByName("map")
    if map:isVisible() == true then
        self:closePannel()
        return
    end
    local index = self._worldUILayer._curMapLayer:getMapResIndex(data.cid)
    if index >= 500 and index < 900 and index ~= 800 then
        cclog("===12=============="..index)
        return
    end
    if data.type == 1 then
        if dd.cfgWorldRes[math.floor(data.avatar/10000)]['type'] == 1 and dd.myinfo:getLong(1,"level") <= 10 then
            dd.mainUI:onShowTips(ff.langString("司令部等级不足"))
            return
        end
        if dd.cfgWorldRes[math.floor(data.avatar/10000)]['type'] == 3 and dd.myinfo:getLong(1,"level") <= 14 then
            dd.mainUI:onShowTips(ff.langString("司令部等级不足"))
            return
        end
    end
    
    map:setTag(1)
    map:getChildByName("map"):setTag(data.cid)

    map:getChildByName("map"):getChildByName("mask"):stopAllActions()
    --zyj 2015-12-1 放大这个控件 适应3d视觉网格
    if dd.openGrid then
        map:getChildByName("map"):getChildByName("mask"):setScale(1.2/dd.NodeGridTable.pro)
    else
        map:getChildByName("map"):getChildByName("mask"):setScale(1.0)
    end
    local action1 = cc.FadeIn:create(1)
    local action2 = cc.FadeOut:create(0.8)
    map:getChildByName("map"):getChildByName("mask"):runAction(cc.RepeatForever:create(cc.Sequence:create(action1,action2)))

    --mask

    local widget = map:getChildByName("map")
    --显示按钮
    widget:getChildByName("f2"):setVisible(false)
    widget:getChildByName("f3"):setVisible(false)
    widget:getChildByName("f4"):setVisible(false)

    widget:getChildByName("f2"):setTag(0)
    widget:getChildByName("f3"):setTag(0)
    widget:getChildByName("f4"):setTag(0)

    cclog("gid:"..data.cid.."=="..data.type.."=="..data.avatar)

    widget:getChildByName("info_btn"):setTag(data.cid)
    self._cache['cid']   = data.cid
    self._cache['uid']   = data.uid
    self._cache['type']  = data.type
    self._cache['res_type']  = data.type
    self._cache['res_id']    = math.floor(data.avatar/10000)

    self._cache['avatar']= data.avatar
    self._cache['level'] = data.level
    self._cache['cur']   = 0
    self._cache['cary']  = 0
    self._cache['unionid']  = data.unionid
    self._cache['uname']    = ""
    self._cache['short']    = data.short
    self._cache['ulevel']   = 0
    self._cache['name']     = data.name
    self._cache['count']    = 0
    self._cache['res']      = 0
    self._cache['friend']   = 0
    self._cache['protect']  = 0
    self._cache['troopid']  = 0
    self._cache['npctype']  = 0

    if showimg == nil then
        showimg = false
    end
    --widget:getChildByName("item"):loadTexture(dd.getPathFromUI("public/alpha.png",1))
    widget:getChildByName("item"):setVisible(showimg)
    widget:getChildByName("info"):getChildByName("icon"):setVisible(false)

    if data.type == 0 then
        widget:getChildByName("count"):setVisible(false)
        widget:getChildByName("progress"):setVisible(false)

        widget:getChildByName("info"):getChildByName("name"):setString(ff.langString("空地"))
        widget:getChildByName("info"):getChildByName("city"):setString(self:getCidStr(data.cid))
        widget:getChildByName("info"):getChildByName("name"):setVisible(true)
        widget:getChildByName("info"):getChildByName("city"):setVisible(true)
        widget:getChildByName("info"):setContentSize(cc.size(widget:getChildByName("info"):getChildByName("city"):getContentSize().width +40,60))
        widget:getChildByName("info"):getChildByName("name"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
        widget:getChildByName("info"):getChildByName("city"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)

        widget:getChildByName("f3"):setTag(2)
        widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("占领"))
        widget:getChildByName("f4"):setTag(5)
        widget:getChildByName("f4"):getChildByName("label"):setString(ff.langString("迁徙"))

        local image = self._worldUILayer._curMapLayer:getCidImage(self._worldUILayer:getCid(data.cid-1))
        self._cache['image']  = image
        self._cache['sharename'] = ff.langString("空地")
        self._cache['sharelevel']= 0  

        --cclog("image:"..image)
        if showimg == true then
            widget:getChildByName("item"):loadTexture(image,1)
        end

        self._cache['type']  = 1
        widget:setTag(data.cid)
    elseif data.type == 1 then
        local avatar = data.avatar
        local image  = dd.cfgWorldRes[math.floor(avatar/10000)]['image']
        local level  = dd.cfgWorldRes[math.floor(avatar/10000)]['level']
        local type   = dd.cfgWorldRes[math.floor(avatar/10000)]['type']
        local name   = ff.langString(dd.cfgWorldRes[math.floor(avatar/10000)]['name'])
        if data.uid > 0 then
            widget:getChildByName("info"):getChildByName("name"):setVisible(true)
            self._cache['sharename'] = name
            self._cache['sharelevel']= level                     

            if data.uid == dd.uid then
                widget:getChildByName("f3"):setTag(6)
                widget:getChildByName("f4"):setTag(9)
                self._cache['troopid'] = 0

                widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("召回"))
                widget:getChildByName("f4"):getChildByName("label"):setString(ff.langString("部队"))
            else            
                local friend = avatar%10
                if friend == 0 or friend == 3 or friend == 4 then
                    widget:getChildByName("f3"):setTag(2)
                    widget:getChildByName("f4"):setTag(3)

                    widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("攻击"))
                    widget:getChildByName("f4"):getChildByName("label"):setString(ff.langString("侦查"))
                    self._cache['type'] = 1 
                end
            end
        else
            self._cache['sharename'] = name
            self._cache['sharelevel']= level                      
            widget:getChildByName("f3"):setTag(11)
            widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("采集"))
            self._cache['type'] = 1 
        end
        widget:getChildByName("info"):getChildByName("name"):setString("???")            
        widget:getChildByName("info"):getChildByName("city"):setString(self:getCidStr(data.cid))

        widget:getChildByName("info"):setContentSize(cc.size(widget:getChildByName("info"):getChildByName("name"):getContentSize().width +120,60))
        widget:getChildByName("info"):getChildByName("name"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
        widget:getChildByName("info"):getChildByName("city"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)


        widget:getChildByName("info"):getChildByName("icon"):setVisible(true)
        widget:getChildByName("info"):getChildByName("icon"):loadTexture(dd.getPathFromUI("public/ic0"..type..".png",1))
        widget:getChildByName("info"):getChildByName("icon"):setPositionX(widget:getChildByName("info"):getChildByName("name"):getPositionX() - widget:getChildByName("info"):getChildByName("name"):getContentSize().width/2 - 30)
        widget:getChildByName("info"):getChildByName("name"):setVisible(true)
        widget:getChildByName("info"):getChildByName("city"):setVisible(true)

        widget:setTag(data.cid)

        self._cache['res'] = type
        self._cache['res_level'] = level

        cclog("image:"..image)
        if showimg == true then
            widget:getChildByName("item"):loadTexture("world/"..image..".png",1)
        end
        --发送查询消息
        local message = ff.net:createMessage()
        message:addCmd(0x0000000c)
        message:addLong(dd.uid)
        message:addLong(0)
        message:addLong(0)
        message:addLong(data.cid)
        cclog("cid:"..data.cid)
        ff.net:rpc(message,false)
        self._cache['image']  = "world/"..image..".png"
    elseif data.type == 2 then
        local avatar = data.avatar        
        local base   = data.level
        local image  = 11
        local level  = math.floor(base/100)
        if base%100 ~= 0 then
            image = base%100
        else    
            if level <= 5 then
                image  = 11
            elseif level <= 10 then
                image = 12
            else
                image = 13
            end
        end    
        self._cache['protect'] = math.floor(avatar/100)
        self._cache['level'] = level
        self._cache['res'] = 7
        self._cache['npctype'] = data.level%100
        widget:getChildByName("info"):getChildByName("name"):setVisible(true)
        widget:getChildByName("info"):getChildByName("city"):setVisible(true)

        widget:getChildByName("info"):getChildByName("city"):setString(self:getCidStr(data.cid))
        
        if data.uid > 0 then
            if data.unionid > 0 then
                widget:getChildByName("info"):getChildByName("name"):setString("["..data.short.."]"..data.name.." LV."..level)            
                self._cache['sharename'] = "["..data.short.."]"..data.name
                self._cache['sharelevel']= level   
            else    
                widget:getChildByName("info"):getChildByName("name"):setString(data.name.." LV."..level)   
                self._cache['sharename'] = data.name
                self._cache['sharelevel']= level   
            end                
            self._cache['share_uname'] = data.name

            if data.uid == dd.uid then
                widget:getChildByName("f2"):setTag(20)
                widget:getChildByName("f2"):getChildByName("label"):setString(ff.langString("皮肤"))

                widget:getChildByName("f3"):setTag(7)
                widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("进入"))

                widget:getChildByName("f4"):setTag(17)
                widget:getChildByName("f4"):getChildByName("label"):setString(ff.langString("增益"))
            else            
                local friend = avatar%10
                --if friend == 4 or friend == 0 then
                if friend == 2 then
                    widget:getChildByName("f3"):setTag(8)
                    widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("增援"))
                    widget:getChildByName("f4"):setTag(18)
                    widget:getChildByName("f4"):getChildByName("label"):setString(ff.langString("运输"))
                else
                    widget:getChildByName("f2"):setTag(1)
                    widget:getChildByName("f3"):setTag(3)
                    widget:getChildByName("f4"):setTag(4)

                    widget:getChildByName("f2"):getChildByName("label"):setString(ff.langString("攻击"))
                    widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("侦查"))
                    widget:getChildByName("f4"):getChildByName("label"):setString(ff.langString("联合攻击"))
                    self._cache['type'] = 2
                end
            end
        end       

        widget:getChildByName("info"):setContentSize(cc.size(widget:getChildByName("info"):getChildByName("name"):getContentSize().width +40,60))
        widget:getChildByName("info"):getChildByName("name"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
        widget:getChildByName("info"):getChildByName("city"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)

        local message = ff.net:createMessage()
        message:addCmd(0x0000000b)
        message:addLong(dd.uid)
        message:addLong(0)
        message:addLong(0)
        message:addLong(data.cid)
        ff.net:rpc(message,false)

        --widget:getChildByName("res"):setVisible(false)
        --widget:getChildByName("count"):setVisible(false)
        --widget:getChildByName("progress"):setVisible(false)
        --widget:getChildByName("progressbar"):setVisible(false)
        widget:setTag(data.cid)

        if showimg == true then
            widget:getChildByName("item"):loadTexture("world/"..image..".png" ,1)
        end
        self._cache['image'] = "world/"..image..".png"        
    elseif data.type == 3 then
        --帐篷
        local level  = math.floor(data.avatar%10000/10)
        local friend = math.floor(data.avatar%10)

        self._cache['res'] = 8

        widget:getChildByName("info"):getChildByName("name"):setVisible(true)
        widget:getChildByName("info"):getChildByName("city"):setVisible(true)

        if data.uid > 0 then
            if data.unionid > 0 then
                widget:getChildByName("info"):getChildByName("name"):setString("["..data.short.."]"..data.name.." Lv."..level)            
                self._cache['sharename'] = "["..data.short.."]"..data.name
                self._cache['sharelevel']= level   
            else    
                widget:getChildByName("info"):getChildByName("name"):setString(data.name.." Lv."..level)   
                self._cache['sharename'] = data.name
                self._cache['sharelevel']= level     
            end                
            self._cache['share_uname'] = data.name
            widget:getChildByName("info"):getChildByName("city"):setString(self:getCidStr(data.cid))

            self._cache['level'] = level
            if data.uid == dd.uid then
                widget:getChildByName("f3"):setTag(6)
                widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("召回"))
                widget:getChildByName("f4"):setTag(9)
                self._cache['troopid'] = 0
                widget:getChildByName("f4"):getChildByName("label"):setString(ff.langString("查看部队"))
            else            
                if friend == 4 or friend == 0 then
                    widget:getChildByName("f3"):setTag(1)
                    widget:getChildByName("f4"):setTag(3)
                    widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("攻击"))
                    widget:getChildByName("f4"):getChildByName("label"):setString(ff.langString("侦查"))
                    self._cache['type'] = 1
                end
                local message = ff.net:createMessage()
                message:addCmd(0x0000000d)
                message:addLong(dd.uid)
                message:addLong(0)
                message:addLong(0)
                message:addLong(data.cid)
                ff.net:rpc(message,false)
            end
        end       
        --widget:getChildByName("res"):setVisible(false)
        --widget:getChildByName("count"):setVisible(false)
        --widget:getChildByName("progress"):setVisible(false)
        --widget:getChildByName("progressbar"):setVisible(false)

        widget:getChildByName("info"):setContentSize(cc.size(widget:getChildByName("info"):getChildByName("name"):getContentSize().width +40,60))
        widget:getChildByName("info"):getChildByName("name"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
        widget:getChildByName("info"):getChildByName("city"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)

        widget:setTag(data.cid)
        self._cache['image'] = "world/800.png"

        if showimg == true then
            widget:getChildByName("item"):loadTexture("world/800.png",1)
        end        
    end            
    --读取icon
    --出现按钮
    local function startAction()
        widget:getChildByName("share"):setVisible(true)
        widget:getChildByName("collect"):setVisible(true)
        widget:getChildByName("info_btn"):setVisible(true)

        widget:getChildByName("share"):setPositionX(145)
        widget:getChildByName("share"):setPositionY(145)
        widget:getChildByName("share"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(30,230))))

        widget:getChildByName("collect"):setPositionX(145)
        widget:getChildByName("collect"):setPositionY(145)
        widget:getChildByName("collect"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(4,145))))

        widget:getChildByName("info_btn"):setPositionX(145)
        widget:getChildByName("info_btn"):setPositionY(145)
        widget:getChildByName("info_btn"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(30,65))))

        widget:getChildByName("back2"):setScale(0.1)
        widget:getChildByName("back2"):runAction(cc.EaseExponentialOut:create(cc.ScaleTo:create(0.5, 1)))

        widget:getChildByName("info"):setScaleX(0.1)
        widget:getChildByName("info"):runAction(cc.EaseExponentialOut:create(cc.ScaleTo:create(0.5, 1)))
        dd.currentLayer._widget:getChildByName("map"):setVisible(true)


        local tick = 0
        for i=1,3 do
            local j = 5 - i
            local tag = widget:getChildByName("f"..j):getTag()
            if tag > 0 then
                local icon = tag
                if icon == 16 then
                    icon = 6
                end
                widget:getChildByName("f"..j):getChildByName("icon"):loadTexture("images/icon/world/ic"..icon.."_map.png")
                widget:getChildByName("f"..j):setVisible(true)
                widget:getChildByName("f"..j):setPositionX(90)
                --widget:getChildByName("f"..j):getChildByName("label"):setPositionY(18)
                widget:getChildByName("f"..j):requestDoLayout()

                local fx = 230
                if j == 3 then
                    fx = 250
                end
                if tick > 0 then
                    widget:getChildByName("f"..j):runAction(cc.Sequence:create(cc.DelayTime:create(tick),cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY())))))
                else
                    widget:getChildByName("f"..j):runAction(cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY()))))
                end
                --tick = tick + 0.1           
            end    
        end
    end       

    widget:stopAllActions()
    --widget:setScale(0.2)
    --widget:setOpacity(0)
    --widget:setPosition(point)

    --根据坐标定位底图

    --widget:setPosition(cc.p(cc.p(ff.width/2,ff.height - 360)))
    widget:setPosition(cc.p(cc.p(ff.width/2,(ff.height - 221)/2 + 221)))
    

    --guide auther:lhz   change fellow code

    GuideControl:GuideCheck()

    --guide end
    performWithDelay(widget, startAction, 0.1)
end
function WorldControl:showStronghold(gid,type)   
    
    AudioEngine.playEffect(ff.sounds[101])
    local gidreal = gid
    for i=1,dd.StrongholdCnt do
        if WorldControl:getInstance()._worldData.Stronghold[i]["gid1"] == gid then
            break
        end
        if WorldControl:getInstance()._worldData.Stronghold[i]["gid2"] == gid or WorldControl:getInstance()._worldData.Stronghold[i]["gid3"] == gid or WorldControl:getInstance()._worldData.Stronghold[i]["gid4"] == gid then
            gidreal = WorldControl:getInstance()._worldData.Stronghold[i]["gid1"]
            break
        end
        if WorldControl:getInstance()._worldData.Stronghold[i]["gid5"] ~= nil then
            if WorldControl:getInstance()._worldData.Stronghold[i]["gid5"] == gid or WorldControl:getInstance()._worldData.Stronghold[i]["gid6"] == gid or WorldControl:getInstance()._worldData.Stronghold[i]["gid7"] == gid or WorldControl:getInstance()._worldData.Stronghold[i]["gid8"] == gid or WorldControl:getInstance()._worldData.Stronghold[i]["gid9"] == gid then
                gidreal = WorldControl:getInstance()._worldData.Stronghold[i]["gid1"]
                break
            end
        end
    end

    cclog("!!!!!!!!!!!!!!!!!显示大圆钮"..type)
    local map = dd.currentLayer._widget:getChildByName("map")
    if map:isVisible() == true then
        self:closePannel()
        return
    end
    local index = self._worldUILayer._curMapLayer:getMapResIndex(gidreal)
    cclog("==================================="..index)
    --if dd.KSRoom == 0 then
        --if index < 800 then
        --    return
        --end
    --end
    
    map:setTag(1)
    map:getChildByName("map"):setTag(gidreal)

    map:getChildByName("map"):getChildByName("mask"):stopAllActions()
    local action1 = cc.FadeIn:create(1)
    local action2 = cc.FadeOut:create(0.8)
    map:getChildByName("map"):getChildByName("mask"):runAction(cc.RepeatForever:create(cc.Sequence:create(action1,action2)))

    --mask

    local widget = map:getChildByName("map")
    --显示按钮
    widget:getChildByName("f2"):setVisible(false)
    widget:getChildByName("f3"):setVisible(false)
    widget:getChildByName("f4"):setVisible(false)

    widget:getChildByName("f2"):setTag(0)
    widget:getChildByName("f3"):setTag(0)
    widget:getChildByName("f4"):setTag(0)

    cclog("gid:"..gid)

    widget:getChildByName("info_btn"):setTag(gidreal)
    self._cache['cid']   = gidreal
    self._cache['id'] = WorldControl:getInstance()._worldData._Stronghold[gidreal]["id"]
 
    self._cache['uid']   = 0
    self._cache['type'] = 23
    self._cache['res_type']  = 0
    self._cache['res_id']    = 0

    self._cache['avatar']= 0
    self._cache['level'] = 0
    self._cache['cur']   = 0
    self._cache['cary']  = 0
    self._cache['unionid']  = 0
    self._cache['uname']    = ""
    self._cache['short']    = ""
    self._cache['ulevel']   = 0
    self._cache['name']     = ""
    self._cache['count']    = 0
    self._cache['res']      = 9
    self._cache['friend']   = 0
    self._cache['protect']  = 0
    self._cache['troopid']  = 0
    self._cache['npctype']  = 0



    widget:getChildByName("item"):setVisible(false)
    widget:getChildByName("info"):getChildByName("icon"):setVisible(false)

    
    widget:getChildByName("count"):setVisible(false)
    widget:getChildByName("progress"):setVisible(false)

    widget:getChildByName("info"):getChildByName("name"):setString(ff.langString("据点"))
    widget:getChildByName("info"):getChildByName("city"):setString(self:getCidStr(gidreal))
    widget:getChildByName("info"):getChildByName("name"):setVisible(true)
    widget:getChildByName("info"):getChildByName("city"):setVisible(true)
    widget:getChildByName("info"):setContentSize(cc.size(widget:getChildByName("info"):getChildByName("city"):getContentSize().width +40,60))
    widget:getChildByName("info"):getChildByName("name"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
    widget:getChildByName("info"):getChildByName("city"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
    

    self._cache['type']  = 23

    if type == 2 then
        if dd.KSRoom == 0 then
            widget:getChildByName("f2"):setTag(22)
        else
            widget:getChildByName("f2"):setTag(29)

        end
        widget:getChildByName("f2"):getChildByName("label"):setString(ff.langString("玩法说明"))
        widget:getChildByName("f3"):setTag(4)
        widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("联合攻击"))
        widget:getChildByName("f4"):setTag(21)
        widget:getChildByName("f4"):getChildByName("label"):setString(ff.langString("信息"))
    else
        widget:getChildByName("f3"):setTag(21)
        widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("信息"))
    end
    


    self._cache['sharename'] = ff.langString("据点")
    self._cache['sharelevel']= 0  

    --cclog("image:"..image)
    if showimg == true then
        widget:getChildByName("item"):loadTexture(image,1)
    end

    widget:setTag(gidreal)

    self._cache['image'] = "world/800.png"
           
    --读取icon
    --出现按钮
    local function startAction()
        widget:getChildByName("share"):setVisible(false)
        widget:getChildByName("collect"):setVisible(false)
        widget:getChildByName("info_btn"):setVisible(true)

        widget:getChildByName("share"):setPositionX(145)
        widget:getChildByName("share"):setPositionY(145)
        widget:getChildByName("share"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(30,230))))

        widget:getChildByName("collect"):setPositionX(145)
        widget:getChildByName("collect"):setPositionY(145)
        widget:getChildByName("collect"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(4,145))))

        widget:getChildByName("info_btn"):setPositionX(145)
        widget:getChildByName("info_btn"):setPositionY(145)
        widget:getChildByName("info_btn"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(30,65))))

        widget:getChildByName("back2"):setScale(0.1)
        widget:getChildByName("back2"):runAction(cc.EaseExponentialOut:create(cc.ScaleTo:create(0.5, 1)))

        widget:getChildByName("info"):setScaleX(0.1)
        widget:getChildByName("info"):runAction(cc.EaseExponentialOut:create(cc.ScaleTo:create(0.5, 1)))
        dd.currentLayer._widget:getChildByName("map"):setVisible(true)


        local tick = 0
        for i=1,3 do
            local j = 5 - i
            local tag = widget:getChildByName("f"..j):getTag()
            if tag > 0 then
                local icon = tag
                if icon == 16 then
                    icon = 6
                end
                widget:getChildByName("f"..j):getChildByName("icon"):loadTexture("images/icon/world/ic"..icon.."_map.png")
                widget:getChildByName("f"..j):setVisible(true)
                widget:getChildByName("f"..j):setPositionX(90)
                --widget:getChildByName("f"..j):getChildByName("label"):setPositionY(18)
                widget:getChildByName("f"..j):requestDoLayout()

                local fx = 230
                if j == 3 then
                    fx = 250
                end
                if tick > 0 then
                    widget:getChildByName("f"..j):runAction(cc.Sequence:create(cc.DelayTime:create(tick),cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY())))))
                else
                    widget:getChildByName("f"..j):runAction(cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY()))))
                end
                --tick = tick + 0.1           
            end    
        end
    end       

    widget:stopAllActions()
    
    widget:setPosition(cc.p(cc.p(ff.width/2,(ff.height - 221)/2 + 221)))
    
    performWithDelay(widget, startAction, 0.1)
end
function WorldControl:showStronghold2(gid,type)   
    
    AudioEngine.playEffect(ff.sounds[101])
    local gidreal = gid
    local id = 0
    for i=1,4 do
        if WorldControl:getInstance()._worldData.Stronghold2[i]["gid1"] == gid then
            id = WorldControl:getInstance()._worldData.Stronghold2[i]["id"]
            break
        end
        if WorldControl:getInstance()._worldData.Stronghold2[i]["gid2"] == gid or WorldControl:getInstance()._worldData.Stronghold2[i]["gid3"] == gid or WorldControl:getInstance()._worldData.Stronghold2[i]["gid4"] == gid then
            gidreal = WorldControl:getInstance()._worldData.Stronghold2[i]["gid1"]
            id = WorldControl:getInstance()._worldData.Stronghold2[i]["id"]
            break
        end
    end

    local map = dd.currentLayer._widget:getChildByName("map")
    if map:isVisible() == true then
        self:closePannel()
        return
    end
    local index = self._worldUILayer._curMapLayer:getMapResIndex(gidreal)

    --if index < 800 then
    --    return
    --end
    
    map:setTag(1)
    map:getChildByName("map"):setTag(gidreal)

    map:getChildByName("map"):getChildByName("mask"):stopAllActions()
    local action1 = cc.FadeIn:create(1)
    local action2 = cc.FadeOut:create(0.8)
    map:getChildByName("map"):getChildByName("mask"):runAction(cc.RepeatForever:create(cc.Sequence:create(action1,action2)))

    --mask

    local widget = map:getChildByName("map")
    --显示按钮
    widget:getChildByName("f2"):setVisible(false)
    widget:getChildByName("f3"):setVisible(false)
    widget:getChildByName("f4"):setVisible(false)

    widget:getChildByName("f2"):setTag(0)
    widget:getChildByName("f3"):setTag(0)
    widget:getChildByName("f4"):setTag(0)

    cclog("gid:"..gid)

    widget:getChildByName("info_btn"):setTag(gidreal)
    self._cache['cid']   = gidreal
    self._cache['id'] = WorldControl:getInstance()._worldData._Stronghold[gidreal]["id"]
 
    self._cache['uid']   = id
    self._cache['type'] = 23
    self._cache['res_type']  = 0
    self._cache['res_id']    = 0

    self._cache['avatar']= 0
    self._cache['level'] = 0
    self._cache['cur']   = 0
    self._cache['cary']  = 0
    self._cache['unionid']  = 0
    self._cache['uname']    = ""
    self._cache['short']    = ""
    self._cache['ulevel']   = 0
    self._cache['name']     = ""
    self._cache['count']    = 0
    self._cache['res']      = 11
    self._cache['friend']   = 0
    self._cache['protect']  = 0
    self._cache['troopid']  = 0
    self._cache['npctype']  = 0



    widget:getChildByName("item"):setVisible(false)
    widget:getChildByName("info"):getChildByName("icon"):setVisible(false)

    
    widget:getChildByName("count"):setVisible(false)
    widget:getChildByName("progress"):setVisible(false)

    widget:getChildByName("info"):getChildByName("name"):setString(ff.langString("据点"))
    widget:getChildByName("info"):getChildByName("city"):setString(self:getCidStr(gidreal))
    widget:getChildByName("info"):getChildByName("name"):setVisible(true)
    widget:getChildByName("info"):getChildByName("city"):setVisible(true)
    widget:getChildByName("info"):setContentSize(cc.size(widget:getChildByName("info"):getChildByName("city"):getContentSize().width +40,60))
    widget:getChildByName("info"):getChildByName("name"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
    widget:getChildByName("info"):getChildByName("city"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)

    if type == 2 then
        widget:getChildByName("f2"):setTag(26)
        widget:getChildByName("f2"):getChildByName("label"):setString(ff.langString("玩法说明"))
        widget:getChildByName("f3"):setTag(4)
        widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("联合攻击"))
  
    elseif type == 1 then
        widget:getChildByName("f2"):setTag(26)
        widget:getChildByName("f2"):getChildByName("label"):setString(ff.langString("玩法说明"))
        widget:getChildByName("f3"):setTag(23)
        widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("点击参加驻防"))
    else
        widget:getChildByName("f2"):setTag(26)
        widget:getChildByName("f2"):getChildByName("label"):setString(ff.langString("玩法说明"))
        widget:getChildByName("f3"):setTag(28)
        widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("进入"))
        widget:getChildByName("f4"):setTag(25)
        widget:getChildByName("f4"):getChildByName("label"):setString(ff.langString("绿卡"))
    end
    


    self._cache['sharename'] = ff.langString("据点")
    self._cache['sharelevel']= 0  

    --cclog("image:"..image)
    if showimg == true then
        widget:getChildByName("item"):loadTexture(image,1)
    end

    widget:setTag(gidreal)

    self._cache['image'] = "images/active/861.png"
           
    --读取icon
    --出现按钮
    local function startAction()
        widget:getChildByName("share"):setVisible(false)
        widget:getChildByName("collect"):setVisible(false)
        widget:getChildByName("info_btn"):setVisible(true)

        widget:getChildByName("share"):setPositionX(145)
        widget:getChildByName("share"):setPositionY(145)
        widget:getChildByName("share"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(30,230))))

        widget:getChildByName("collect"):setPositionX(145)
        widget:getChildByName("collect"):setPositionY(145)
        widget:getChildByName("collect"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(4,145))))

        widget:getChildByName("info_btn"):setPositionX(145)
        widget:getChildByName("info_btn"):setPositionY(145)
        widget:getChildByName("info_btn"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(30,65))))

        widget:getChildByName("back2"):setScale(0.1)
        widget:getChildByName("back2"):runAction(cc.EaseExponentialOut:create(cc.ScaleTo:create(0.5, 1)))

        widget:getChildByName("info"):setScaleX(0.1)
        widget:getChildByName("info"):runAction(cc.EaseExponentialOut:create(cc.ScaleTo:create(0.5, 1)))
        dd.currentLayer._widget:getChildByName("map"):setVisible(true)


        local tick = 0
        for i=1,3 do
            local j = 5 - i
            local tag = widget:getChildByName("f"..j):getTag()
            if tag > 0 then
                local icon = tag
                if icon == 16 then
                    icon = 6
                end
                widget:getChildByName("f"..j):getChildByName("icon"):loadTexture("images/icon/world/ic"..icon.."_map.png")
                widget:getChildByName("f"..j):setVisible(true)
                widget:getChildByName("f"..j):setPositionX(90)
                --widget:getChildByName("f"..j):getChildByName("label"):setPositionY(18)
                widget:getChildByName("f"..j):requestDoLayout()

                local fx = 230
                if j == 3 then
                    fx = 250
                end
                if tick > 0 then
                    widget:getChildByName("f"..j):runAction(cc.Sequence:create(cc.DelayTime:create(tick),cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY())))))
                else
                    widget:getChildByName("f"..j):runAction(cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY()))))
                end
                --tick = tick + 0.1           
            end    
        end
    end       

    widget:stopAllActions()
    
    widget:setPosition(cc.p(cc.p(ff.width/2,(ff.height - 221)/2 + 221)))
    
    performWithDelay(widget, startAction, 0.1)
end
function WorldControl:showTower(gid,type)   
    
    AudioEngine.playEffect(ff.sounds[101])
    if WorldControl:getInstance()._worldData._Tower[gid] == nil then
        return
    end
    local map = dd.currentLayer._widget:getChildByName("map")
    if map:isVisible() == true then
        self:closePannel()
        return
    end
    local index = self._worldUILayer._curMapLayer:getMapResIndex(gid)
    if index ~= 702 then
        return
    end

    local id = 0
    for i=1,4 do
        if WorldControl:getInstance()._worldData.Tower[i]["gid1"] == gid then
            id = WorldControl:getInstance()._worldData.Tower[i]["id"]
            break
        end
        if WorldControl:getInstance()._worldData.Tower[i]["gid2"] == gid or WorldControl:getInstance()._worldData.Tower[i]["gid3"] == gid or WorldControl:getInstance()._worldData.Tower[i]["gid4"] == gid then
            gidreal = WorldControl:getInstance()._worldData.Tower[i]["gid1"]
            id = WorldControl:getInstance()._worldData.Tower[i]["id"]
            break
        end
    end
    
    map:setTag(1)
    map:getChildByName("map"):setTag(gid)

    map:getChildByName("map"):getChildByName("mask"):stopAllActions()
    local action1 = cc.FadeIn:create(1)
    local action2 = cc.FadeOut:create(0.8)
    map:getChildByName("map"):getChildByName("mask"):runAction(cc.RepeatForever:create(cc.Sequence:create(action1,action2)))

    --mask

    local widget = map:getChildByName("map")
    --显示按钮
    widget:getChildByName("f2"):setVisible(false)
    widget:getChildByName("f3"):setVisible(false)
    widget:getChildByName("f4"):setVisible(false)

    widget:getChildByName("f2"):setTag(0)
    widget:getChildByName("f3"):setTag(0)
    widget:getChildByName("f4"):setTag(0)

    cclog("gid:"..gid)

    widget:getChildByName("info_btn"):setTag(gid)
    self._cache['cid']   = gid
    self._cache['id'] = id
 
    self._cache['uid']   = 0
    self._cache['type'] = 23
    self._cache['res_type']  = 0
    self._cache['res_id']    = 0

    self._cache['avatar']= 0
    self._cache['level'] = 0
    self._cache['cur']   = 0
    self._cache['cary']  = 0
    self._cache['unionid']  = 0
    self._cache['uname']    = ""
    self._cache['short']    = ""
    self._cache['ulevel']   = 0
    self._cache['name']     = ""
    self._cache['count']    = 0
    self._cache['res']      = 10
    self._cache['friend']   = 0
    self._cache['protect']  = 0
    self._cache['troopid']  = 0
    self._cache['npctype']  = 0



    widget:getChildByName("item"):setVisible(false)
    widget:getChildByName("info"):getChildByName("icon"):setVisible(false)

    
    widget:getChildByName("count"):setVisible(false)
    widget:getChildByName("progress"):setVisible(false)

    widget:getChildByName("info"):getChildByName("name"):setString(ff.langString("哨塔"))
    widget:getChildByName("info"):getChildByName("city"):setString(self:getCidStr(gid))
    widget:getChildByName("info"):getChildByName("name"):setVisible(true)
    widget:getChildByName("info"):getChildByName("city"):setVisible(true)
    widget:getChildByName("info"):setContentSize(cc.size(widget:getChildByName("info"):getChildByName("city"):getContentSize().width +40,60))
    widget:getChildByName("info"):getChildByName("name"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
    widget:getChildByName("info"):getChildByName("city"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
    

    if type == 2 then
        
        widget:getChildByName("f3"):setTag(27)
        widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("进入"))
  
    elseif type == 1 then
        widget:getChildByName("f3"):setTag(4)
        widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("联合攻击"))
        widget:getChildByName("f4"):setTag(27)
        widget:getChildByName("f4"):getChildByName("label"):setString(ff.langString("信息"))
    else
        widget:getChildByName("f3"):setTag(27)
        widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("信息"))
        
    end
       
    --cclog("image:"..image)
    if showimg == true then
        widget:getChildByName("item"):loadTexture(image,1)
    end
    self._cache['sharename'] = ff.langString("哨塔")
    self._cache['sharelevel']= 0  

    --cclog("image:"..image)
    

    widget:setTag(gid)

           
    --读取icon
    --出现按钮
    local function startAction()
        widget:getChildByName("share"):setVisible(false)
        widget:getChildByName("collect"):setVisible(false)
        widget:getChildByName("info_btn"):setVisible(true)

        widget:getChildByName("share"):setPositionX(145)
        widget:getChildByName("share"):setPositionY(145)
        widget:getChildByName("share"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(30,230))))

        widget:getChildByName("collect"):setPositionX(145)
        widget:getChildByName("collect"):setPositionY(145)
        widget:getChildByName("collect"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(4,145))))

        widget:getChildByName("info_btn"):setPositionX(145)
        widget:getChildByName("info_btn"):setPositionY(145)
        widget:getChildByName("info_btn"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(30,65))))

        widget:getChildByName("back2"):setScale(0.1)
        widget:getChildByName("back2"):runAction(cc.EaseExponentialOut:create(cc.ScaleTo:create(0.5, 1)))

        widget:getChildByName("info"):setScaleX(0.1)
        widget:getChildByName("info"):runAction(cc.EaseExponentialOut:create(cc.ScaleTo:create(0.5, 1)))
        dd.currentLayer._widget:getChildByName("map"):setVisible(true)


        local tick = 0
        for i=1,3 do
            local j = 5 - i
            local tag = widget:getChildByName("f"..j):getTag()
            if tag > 0 then
                local icon = tag
                if icon == 16 then
                    icon = 6
                end
                widget:getChildByName("f"..j):getChildByName("icon"):loadTexture("images/icon/world/ic"..icon.."_map.png")
                widget:getChildByName("f"..j):setVisible(true)
                widget:getChildByName("f"..j):setPositionX(90)
                --widget:getChildByName("f"..j):getChildByName("label"):setPositionY(18)
                widget:getChildByName("f"..j):requestDoLayout()

                local fx = 230
                if j == 3 then
                    fx = 250
                end
                if tick > 0 then
                    widget:getChildByName("f"..j):runAction(cc.Sequence:create(cc.DelayTime:create(tick),cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY())))))
                else
                    widget:getChildByName("f"..j):runAction(cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY()))))
                end
                --tick = tick + 0.1           
            end    
        end
    end       

    widget:stopAllActions()
    
    widget:setPosition(cc.p(cc.p(ff.width/2,(ff.height - 221)/2 + 221)))
    
    performWithDelay(widget, startAction, 0.1)
end
function WorldControl:showStrongSpace(gid,type)   

    AudioEngine.playEffect(ff.sounds[101])

    cclog("显示大圆钮")
    local map = dd.currentLayer._widget:getChildByName("map")
    if map:isVisible() == true then
        self:closePannel()
        return
    end
    local index = self._worldUILayer._curMapLayer:getMapResIndex(gid)
    if index ~= 701 then
        cclog("===12=============="..index)
        return
    end
    
    map:setTag(1)
    map:getChildByName("map"):setTag(gid)

    map:getChildByName("map"):getChildByName("mask"):stopAllActions()
    --zyj 2015-12-1 放大这个控件 适应3d视觉网格
    if dd.openGrid then
        map:getChildByName("map"):getChildByName("mask"):setScale(1.2/dd.NodeGridTable.pro)
    else
        map:getChildByName("map"):getChildByName("mask"):setScale(1.0)
    end
    local action1 = cc.FadeIn:create(1)
    local action2 = cc.FadeOut:create(0.8)
    map:getChildByName("map"):getChildByName("mask"):runAction(cc.RepeatForever:create(cc.Sequence:create(action1,action2)))

    --mask

    local widget = map:getChildByName("map")
    --显示按钮
    widget:getChildByName("f2"):setVisible(false)
    widget:getChildByName("f3"):setVisible(false)
    widget:getChildByName("f4"):setVisible(false)

    widget:getChildByName("f2"):setTag(0)
    widget:getChildByName("f3"):setTag(0)
    widget:getChildByName("f4"):setTag(0)

   

    widget:getChildByName("info_btn"):setTag(gid)
    self._cache['cid']   = gid
    self._cache['uid']   = 0
    self._cache['type']  = 0
    self._cache['res_type']  = 0
    self._cache['res_id']    = 0

    self._cache['avatar']= 0
    self._cache['level'] = 0
    self._cache['cur']   = 0
    self._cache['cary']  = 0
    self._cache['unionid']  = 0
    self._cache['uname']    = ""
    self._cache['short']    = 0
    self._cache['ulevel']   = 0
    self._cache['name']     = ""
    self._cache['count']    = 0
    self._cache['res']      = 0
    self._cache['friend']   = 0
    self._cache['protect']  = 0
    self._cache['troopid']  = 0
    self._cache['npctype']  = 0


    --widget:getChildByName("item"):loadTexture(dd.getPathFromUI("public/alpha.png",1))
    widget:getChildByName("item"):setVisible(false)
    widget:getChildByName("info"):getChildByName("icon"):setVisible(false)


        widget:getChildByName("count"):setVisible(false)
        widget:getChildByName("progress"):setVisible(false)

        widget:getChildByName("info"):getChildByName("name"):setString(ff.langString("空地"))
        widget:getChildByName("info"):getChildByName("city"):setString(self:getCidStr(gid))
        widget:getChildByName("info"):getChildByName("name"):setVisible(true)
        widget:getChildByName("info"):getChildByName("city"):setVisible(true)
        widget:getChildByName("info"):setContentSize(cc.size(widget:getChildByName("info"):getChildByName("city"):getContentSize().width +40,60))
        widget:getChildByName("info"):getChildByName("name"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
        widget:getChildByName("info"):getChildByName("city"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)

        widget:getChildByName("f3"):setTag(24)
        widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("迁徙"))

        local image = self._worldUILayer._curMapLayer:getCidImage(self._worldUILayer:getCid(gid-1))
        self._cache['image']  = image
        self._cache['sharename'] = ff.langString("空地")
        self._cache['sharelevel']= 0  

        --cclog("image:"..image)
        if showimg == true then
            widget:getChildByName("item"):loadTexture(image,1)
        end

        self._cache['type']  = 1
        widget:setTag(gid)
         
    --读取icon
    --出现按钮
    local function startAction()
        widget:getChildByName("share"):setVisible(true)
        widget:getChildByName("collect"):setVisible(true)
        widget:getChildByName("info_btn"):setVisible(true)

        widget:getChildByName("share"):setPositionX(145)
        widget:getChildByName("share"):setPositionY(145)
        widget:getChildByName("share"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(30,230))))

        widget:getChildByName("collect"):setPositionX(145)
        widget:getChildByName("collect"):setPositionY(145)
        widget:getChildByName("collect"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(4,145))))

        widget:getChildByName("info_btn"):setPositionX(145)
        widget:getChildByName("info_btn"):setPositionY(145)
        widget:getChildByName("info_btn"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(30,65))))

        widget:getChildByName("back2"):setScale(0.1)
        widget:getChildByName("back2"):runAction(cc.EaseExponentialOut:create(cc.ScaleTo:create(0.5, 1)))

        widget:getChildByName("info"):setScaleX(0.1)
        widget:getChildByName("info"):runAction(cc.EaseExponentialOut:create(cc.ScaleTo:create(0.5, 1)))
        dd.currentLayer._widget:getChildByName("map"):setVisible(true)


        local tick = 0
        for i=1,3 do
            local j = 5 - i
            local tag = widget:getChildByName("f"..j):getTag()
            if tag > 0 then
                local icon = tag
                if icon == 16 then
                    icon = 6
                end
                widget:getChildByName("f"..j):getChildByName("icon"):loadTexture("images/icon/world/ic"..icon.."_map.png")
                widget:getChildByName("f"..j):setVisible(true)
                widget:getChildByName("f"..j):setPositionX(90)
                --widget:getChildByName("f"..j):getChildByName("label"):setPositionY(18)
                widget:getChildByName("f"..j):requestDoLayout()

                local fx = 230
                if j == 3 then
                    fx = 250
                end
                if tick > 0 then
                    widget:getChildByName("f"..j):runAction(cc.Sequence:create(cc.DelayTime:create(tick),cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY())))))
                else
                    widget:getChildByName("f"..j):runAction(cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY()))))
                end
                --tick = tick + 0.1           
            end    
        end
    end       

    widget:stopAllActions()
    --widget:setScale(0.2)
    --widget:setOpacity(0)
    --widget:setPosition(point)

    --根据坐标定位底图

    --widget:setPosition(cc.p(cc.p(ff.width/2,ff.height - 360)))
    widget:setPosition(cc.p(cc.p(ff.width/2,(ff.height - 221)/2 + 221)))
    

    --guide auther:lhz   change fellow code

    GuideControl:GuideCheck()

    --guide end
    performWithDelay(widget, startAction, 0.1)
end

function WorldControl:updatePannel(message)
    local cid     = message:getLong()
    local unionid = message:getLong()
    local uname   = message:getString(15)
    local short   = message:getString(3)
    local uid     = message:getLong()
    local name    = message:getString(15)
    local face    = message:getLong()
    local ulevel  = message:getLong()
    local vip     = message:getLong()
    local power   = message:getLong()
    local kills   = message:getLong()

    local type    = message:getLong()
    local level   = message:getLong()
    local cur     = message:getLong()
    local speed   = message:getLong()
    local carry   = message:getLong()
    local gets    = message:getLong()

    self._cache['cur']  = cur
    self._cache['gets'] = gets
    self._cache['carry']= carry
    self._cache['speed']= speed
    self._cache['unionid'] = unionid
    self._cache['short']=short
    self._cache['name']=name

    self._cache['own_uname'] = uname

    local widget = dd.currentLayer._widget:getChildByName("map"):getChildByName("map")
    widget:getChildByName("progress"):stopAllActions()

    if widget:getTag() == cid then
        local total = dd.cfgWorldRes[math.floor(self._cache['avatar']/10000)]['count']
        local prog = math.ceil(cur * 100 /total)
        if prog < 0 then
            prog = 0
        end
        local curprogress = 0
        local function setProgress()
            if self._cache['tick'] ~= nil then
                local gets = math.floor((dd.getServerTime() - self._cache['tick']) * self._cache['speed']/3600)
                cclog("gets:"..gets..",time:"..dd.getServerTime() - self._cache['tick']..",speed:"..self._cache['speed']..",carry:"..self._cache['carry']..",cur"..self._cache['cur'])
                if self._cache['cur'] - gets <= 0 then
                    widget:getChildByName("info"):getChildByName("name"):setString(0) 
                    widget:getChildByName("progress"):stopAllActions()
                    return
                end
                widget:getChildByName("info"):getChildByName("name"):setString(ff.getFormatMoney(self._cache['cur'] - gets)) 
            end                
        end


        
        --widget:getChildByName("info"):getChildByName("name"):setString(dd.showKM(cur,5))         
        widget:getChildByName("info"):getChildByName("name"):setString(ff.getFormatMoney(cur))         
        widget:getChildByName("info"):setContentSize(cc.size(widget:getChildByName("info"):getChildByName("name"):getContentSize().width +120,60))
        widget:getChildByName("info"):getChildByName("name"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
        widget:getChildByName("info"):getChildByName("city"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)

        widget:getChildByName("info"):getChildByName("icon"):setVisible(true)
        widget:getChildByName("info"):getChildByName("icon"):loadTexture(dd.getPathFromUI("public/ic0"..type..".png",1))
        widget:getChildByName("info"):getChildByName("icon"):setPositionX(widget:getChildByName("info"):getChildByName("name"):getPositionX() - widget:getChildByName("info"):getChildByName("name"):getContentSize().width/2 - 30)

        self._cache['tick'] = dd.getServerTime()
        if uid > 0 and self._cache['res'] >= 1 and self._cache['res'] <= 6 then
            setProgress()
            schedule(widget:getChildByName("progress"), setProgress,1)
        end

        --resinfo2
        view = dd.currentLayer._widget:getChildByName("resinfo2")
        if unionid > 0 then
            view:getChildByName("detail"):getChildByName("info"):getChildByName("name"):setString("["..short.."]"..name)
        else
            view:getChildByName("detail"):getChildByName("info"):getChildByName("name"):setString(name)
        end                
        view:getChildByName("detail"):getChildByName("info"):getChildByName("union"):setString(uname)
        view:getChildByName("detail"):getChildByName("info"):getChildByName("power"):setString(ff.getFormatMoney(power))
    end
end
function WorldControl:updateCityPannel(message)
    cclog("update city pannel")
    local cid     = message:getLong()
    local unionid = message:getLong()
    local uname   = message:getString(15)
    local short   = message:getString(3)
    local uid     = message:getLong()
    local name    = message:getString(15)
    local face    = message:getLong()
    local ulevel  = message:getLong()
    local vip     = message:getLong()
    local power   = message:getLong()
    local kills   = message:getLong()

    local cityname= message:getString(15)
    local level   = message:getLong()



    if self._cache['cid'] == cid then
        self._cache['face'] = face
        self._cache['short']= short
        self._cache['power']= power
        self._cache['name'] = name
        self._cache['unionid'] = unionid
        self._cache['uname'] = uname
    end

    local widget = dd.currentLayer._widget:getChildByName("map"):getChildByName("map")
    if self._cache['cid'] == cid then
        if string.len(short) == 3 then
            widget:getChildByName("info"):getChildByName("name"):setString("["..short.."]"..name.." LV."..level)            
        else    
            widget:getChildByName("info"):getChildByName("name"):setString(name.." LV."..level)            
        end                
        cclog("cityname:"..cityname)
        --if string.len(cityname) <= 0 then
        --    widget:getChildByName("info"):getChildByName("city"):setString(name)            
        --else
        --    widget:getChildByName("info"):getChildByName("city"):setString(cityname)            
        --end
    end
end
function WorldControl:updateCampPannel(message)
    local cid     = message:getLong()
    local unionid = message:getLong()
    local uname   = message:getString(15)
    local short   = message:getString(3)
    local uid     = message:getLong()
    local name    = message:getString(15)
    local face    = message:getLong()
    local ulevel  = message:getLong()
    local vip     = message:getLong()
    local power   = message:getLong()
    local kills   = message:getLong()

    if self._cache['cid'] == cid then
        self._cache['face'] = face
        self._cache['short']= short
        self._cache['power']= power
        self._cache['name'] = name
        self._cache['unionid'] = unionid
        self._cache['uname'] = uname
    end
end

function WorldControl:onShowTroop(troop)
    AudioEngine.playEffect(ff.sounds[207])
    local auto =  math.floor(math.random() * 1000000)%2
    if auto == 0 then
        AudioEngine.playEffect(ff.sounds["p004"])
    else
        AudioEngine.playEffect(ff.sounds["p005"])
    end

    
    local id = troop:getLong(1,"id")
    local uid = troop:getLong(1,"uid")
    local state = troop:getLong(1,"state")
    local from  = troop:getLong(1,"from")
    local to  = troop:getLong(1,"to")
    local unionid = troop:getLong(1,"unionid")
    local name = troop:getString(1,"name")
    local short = troop:getString(1,"short")    
    local type = troop:getLong(1,"type")

    self._cache['from'] = from
    self._cache['to']   = to
    self._cache['state']   = state
    self._cache['cid']     = id

    self._worldUILayer._flowid = id
    self._worldUILayer._scroll:stopAutoScroll()

    cclog("选中了行进中部队:"..id..",uid:"..uid..",to:"..to)

    local map = dd.currentLayer._widget:getChildByName("map")
    if map:isVisible() == true then
        self:closePannel()
        return
    end
    map:getChildByName("map"):getChildByName("mask"):stopAllActions()
    local action = cc.Blink:create(3, 3)
    map:getChildByName("map"):getChildByName("mask"):runAction(cc.RepeatForever:create(cc.Sequence:create(action,action:reverse())))


    local widget = map:getChildByName("map")
    map:setTag(2)
    map:getChildByName("map"):setTag(id)
    --显示按钮
    widget:getChildByName("f2"):setVisible(false)
    widget:getChildByName("f3"):setVisible(false)
    widget:getChildByName("f4"):setVisible(false)

    widget:getChildByName("f2"):setTag(0)
    widget:getChildByName("f3"):setTag(0)
    widget:getChildByName("f4"):setTag(0)

    --widget:getChildByName("res"):setVisible(false)
    --widget:getChildByName("count"):setVisible(false)
    --widget:getChildByName("progress"):setVisible(false)
    --widget:getChildByName("progressbar"):setVisible(false)

    --widget:getChildByName("progress"):setString(self:getCidStr(to))
    --widget:getChildByName("progress"):setVisible(true)

    widget:getChildByName("info"):getChildByName("name"):setVisible(true)
    widget:getChildByName("info"):getChildByName("city"):setVisible(true)
    --widget:getChildByName("info"):getChildByName("city"):setPositionY(55)
    --widget:getChildByName("info"):getChildByName("cid"):setPositionY(25)

    self._cache['troopid'] = id
    if uid == dd.uid then
        if state == 1 then
            local showback = true
            for i=1,dd.myTroops:rowCount() do
                if dd.myTroops:getLong(i,"id") == id then
                    if dd.myTroops:getLong(i,"type") == 11 then
                        showback = false
                        break
                    end
                end
            end
            widget:getChildByName("f2"):setVisible(showback)
            widget:getChildByName("f2"):setTag(16)
            widget:getChildByName("f2"):getChildByName("label"):setString(ff.langString("回城"))
            widget:getChildByName("f3"):setTag(10)
            widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("加速"))
        else
            if state == 2 then
                widget:getChildByName("f3"):setTag(10)
                widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("加速"))
            end
        end    
        widget:getChildByName("f4"):setTag(9)        
        widget:getChildByName("f4"):getChildByName("label"):setString(ff.langString("查看"))

        if dd.myUnion:getLong(1,"unionid") > 0 then
            widget:getChildByName("info"):getChildByName("name"):setString("["..dd.myUnion:getString(1,"unionflag").."]"..dd.myinfo:getString(1,"name"))
        else
            widget:getChildByName("info"):getChildByName("name"):setString(dd.myinfo:getString(1,"name"))
        end
    else
        if unionid > 0 then
            widget:getChildByName("info"):getChildByName("name"):setString("["..short.."]"..name)
        else
            widget:getChildByName("info"):getChildByName("name"):setString(name)
        end
    end
    dd.currentLayer._widget:getChildByName("resinfo3"):getChildByName("title"):setString(ff.langString("{0}的部队",{name}))
    dd.currentLayer._widget:getChildByName("resinfo3"):setTag(id)

    if type == 1 or type == 2 or type == 4 then
        if state == 1 then
            dd.currentLayer._widget:getChildByName("resinfo3"):getChildByName("face"):getChildByName("face"):loadTexture("images/icon/que/101.png",0)
        else            
            dd.currentLayer._widget:getChildByName("resinfo3"):getChildByName("face"):getChildByName("face"):loadTexture("images/icon/que/102.png",0)
        end
    elseif type == 3 then
        if state == 1 then
            dd.currentLayer._widget:getChildByName("resinfo3"):getChildByName("face"):getChildByName("face"):loadTexture("images/icon/que/201.png",0)
        else            
            dd.currentLayer._widget:getChildByName("resinfo3"):getChildByName("face"):getChildByName("face"):loadTexture("images/icon/que/202.png",0)
        end
    elseif type == 5 then
        if state == 1 then
            dd.currentLayer._widget:getChildByName("resinfo3"):getChildByName("face"):getChildByName("face"):loadTexture("images/icon/que/241.png",0)
        else            
            dd.currentLayer._widget:getChildByName("resinfo3"):getChildByName("face"):getChildByName("face"):loadTexture("images/icon/que/242.png",0)
        end
    elseif type == 11 then
        dd.currentLayer._widget:getChildByName("resinfo3"):getChildByName("face"):getChildByName("face"):loadTexture("images/icon/que/441.png",0)
    end

    widget:getChildByName("info"):getChildByName("city"):setString(self:getCidStr(to))
    widget:getChildByName("info"):setContentSize(cc.size(widget:getChildByName("info"):getChildByName("name"):getContentSize().width +40,60))
    widget:getChildByName("info"):getChildByName("name"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
    widget:getChildByName("info"):getChildByName("city"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
    widget:getChildByName("info"):getChildByName("icon"):setVisible(false)

    widget:getChildByName("item"):setVisible(false)

    widget:getChildByName("info_btn"):setTag(id)
    --出现按钮
    local function startAction()
        widget:getChildByName("share"):setVisible(false)
        widget:getChildByName("collect"):setVisible(false)


        widget:getChildByName("info_btn"):setPositionX(145)
        widget:getChildByName("info_btn"):setPositionY(145)
        widget:getChildByName("info_btn"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(30,65))))

        widget:getChildByName("back2"):setScale(0.1)
        widget:getChildByName("back2"):runAction(cc.ScaleTo:create(0.5, 1))

        widget:getChildByName("info"):setScaleX(0.1)
        widget:getChildByName("info"):runAction(cc.EaseExponentialOut:create(cc.ScaleTo:create(0.5, 1)))
        dd.currentLayer._widget:getChildByName("map"):setVisible(true)

        local tick = 0
        for i=1,3 do
            local j = 5 - i
            local tag = widget:getChildByName("f"..j):getTag()
            if tag > 0 then
                local icon = tag
                if icon == 16 then
                    icon = 6
                end
                widget:getChildByName("f"..j):getChildByName("icon"):loadTexture("images/icon/world/ic"..icon.."_map.png")
                widget:getChildByName("f"..j):setVisible(true)
                widget:getChildByName("f"..j):setPositionX(90)
                --widget:getChildByName("f"..j):getChildByName("label"):setPositionY(18)
                widget:getChildByName("f"..j):requestDoLayout()

                local fx = 230
                if j == 3 then
                    fx = 250
                end
                if tick > 0 then
                    widget:getChildByName("f"..j):runAction(cc.Sequence:create(cc.DelayTime:create(tick),cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY())))))
                else
                    widget:getChildByName("f"..j):runAction(cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY()))))
                end
                --tick = tick + 0.1           
            end    
        end
    end       


    widget:stopAllActions()
    widget:setPosition(cc.p(cc.p(ff.width/2,(ff.height - 221)/2 + 221)))
    performWithDelay(widget, startAction, 0.1)
end

function WorldControl:onShowTroopEx(troop)
    AudioEngine.playEffect(ff.sounds[207])
    local auto =  math.floor(math.random() * 1000000)%2
    if auto == 0 then
        AudioEngine.playEffect(ff.sounds["p004"])
    else
        AudioEngine.playEffect(ff.sounds["p005"])
    end

    
    local id = troop:getLong(1,"id")
    local uid = troop:getLong(1,"uid")
    local state = troop:getLong(1,"state")
    local from  = troop:getLong(1,"from")
    local to  = troop:getLong(1,"to")
    local unionid = troop:getLong(1,"unionid")
    local name = troop:getString(1,"name")
    local short = troop:getString(1,"short")    
    local type = troop:getLong(1,"type")

    self._cache['from'] = from
    self._cache['to']   = to
    self._cache['state']   = state
    self._cache['cid']     = id

    self._worldUILayer._flowid = id
    self._worldUILayer._scroll:stopAutoScroll()

    cclog("选中了行3333333333333进中部队:"..id..",uid:"..uid..",to:"..to)

    local map = dd.currentLayer._widget:getChildByName("map")
    if map:isVisible() == true then
        self:closePannel()
        return
    end
    map:getChildByName("map"):getChildByName("mask"):stopAllActions()
    local action = cc.Blink:create(3, 3)
    map:getChildByName("map"):getChildByName("mask"):runAction(cc.RepeatForever:create(cc.Sequence:create(action,action:reverse())))


    local widget = map:getChildByName("map")
    map:setTag(2)
    map:getChildByName("map"):setTag(id)
    --显示按钮
    widget:getChildByName("f2"):setVisible(false)
    widget:getChildByName("f3"):setVisible(false)
    widget:getChildByName("f4"):setVisible(false)

    widget:getChildByName("f2"):setTag(0)
    widget:getChildByName("f3"):setTag(0)
    widget:getChildByName("f4"):setTag(0)

    widget:getChildByName("info"):getChildByName("name"):setVisible(true)
    widget:getChildByName("info"):getChildByName("city"):setVisible(true)
    widget:getChildByName("info"):getChildByName("name"):setString(ff.langString("地下城堡"))
    widget:getChildByName("info"):getChildByName("icon"):setVisible(false)
    widget:getChildByName("info"):getChildByName("city"):setString(self:getCidStr(to))
    widget:getChildByName("info"):setContentSize(cc.size(widget:getChildByName("info"):getChildByName("name"):getContentSize().width +40,60))
    widget:getChildByName("info"):getChildByName("name"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
    widget:getChildByName("info"):getChildByName("city"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)

    self._cache['troopid'] = id
    if uid == dd.uid then
        
        widget:getChildByName("f3"):setVisible(showback)
        widget:getChildByName("f3"):setTag(16)
        widget:getChildByName("f3"):getChildByName("label"):setString(ff.langString("回城"))
            
        widget:getChildByName("f4"):setTag(9)        
        widget:getChildByName("f4"):getChildByName("label"):setString(ff.langString("查看"))

       
    end
    dd.currentLayer._widget:getChildByName("resinfo3"):getChildByName("title"):setString(ff.langString("{0}的部队",{name}))
    dd.currentLayer._widget:getChildByName("resinfo3"):setTag(id)

   
    dd.currentLayer._widget:getChildByName("resinfo3"):getChildByName("face"):getChildByName("face"):loadTexture("images/icon/que/Exploring.png",0)
        

    widget:getChildByName("info"):getChildByName("city"):setString(self:getCidStr(to))
    widget:getChildByName("info"):setContentSize(cc.size(widget:getChildByName("info"):getChildByName("name"):getContentSize().width +40,60))
    widget:getChildByName("info"):getChildByName("name"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
    widget:getChildByName("info"):getChildByName("city"):setPositionX(widget:getChildByName("info"):getContentSize().width/2)
    widget:getChildByName("info"):getChildByName("icon"):setVisible(false)

    widget:getChildByName("item"):setVisible(false)

    widget:getChildByName("info_btn"):setTag(id)
    --出现按钮
    local function startAction()
        widget:getChildByName("share"):setVisible(false)
        widget:getChildByName("collect"):setVisible(false)


        widget:getChildByName("info_btn"):setPositionX(145)
        widget:getChildByName("info_btn"):setPositionY(145)
        widget:getChildByName("info_btn"):runAction(cc.EaseBackOut:create(cc.MoveTo:create(0.5, cc.p(30,65))))

        widget:getChildByName("back2"):setScale(0.1)
        widget:getChildByName("back2"):runAction(cc.ScaleTo:create(0.5, 1))

        widget:getChildByName("info"):setScaleX(0.1)
        widget:getChildByName("info"):runAction(cc.EaseExponentialOut:create(cc.ScaleTo:create(0.5, 1)))
        dd.currentLayer._widget:getChildByName("map"):setVisible(true)

        local tick = 0
        for i=1,3 do
            local j = 5 - i
            local tag = widget:getChildByName("f"..j):getTag()
            if tag > 0 then
                local icon = tag
                if icon == 16 then
                    icon = 6
                end
                widget:getChildByName("f"..j):getChildByName("icon"):loadTexture("images/icon/world/ic"..icon.."_map.png")
                widget:getChildByName("f"..j):setVisible(true)
                widget:getChildByName("f"..j):setPositionX(90)
                --widget:getChildByName("f"..j):getChildByName("label"):setPositionY(18)
                widget:getChildByName("f"..j):requestDoLayout()

                local fx = 230
                if j == 3 then
                    fx = 250
                end
                if tick > 0 then
                    widget:getChildByName("f"..j):runAction(cc.Sequence:create(cc.DelayTime:create(tick),cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY())))))
                else
                    widget:getChildByName("f"..j):runAction(cc.EaseExponentialOut:create(cc.MoveTo:create(0.5, cc.p(fx,widget:getChildByName("f"..j):getPositionY()))))
                end
                --tick = tick + 0.1           
            end    
        end
    end       


    widget:stopAllActions()
    widget:setPosition(cc.p(cc.p(ff.width/2,(ff.height - 221)/2 + 221)))
    performWithDelay(widget, startAction, 0.1)
end


--添加收藏
function WorldControl.onBook(sender,eventType)
    if eventType==ccui.TouchEventType.ended then     

        dd.mainUI:getChildByName("share"):setVisible(false)

        dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("detail"):getChildByName("info"):getChildByName("pg"):getChildByName("progress"):stopAllActions()
        dd.currentLayer._widget:getChildByName("resinfo1"):setVisible(false)
        dd.currentLayer._widget:getChildByName("resinfo2"):setVisible(false)
        dd.currentLayer._widget:getChildByName("resinfo3"):setVisible(false)
        pop.onShowPopDialog("colPop",WorldControl:getInstance()._cache,WorldControl:getInstance())
    end
end
function WorldControl.onShare(sender,eventType)
    if eventType==ccui.TouchEventType.ended then 
        if dd.currentLayer._widget ~= nil then
            local widget = WorldControl:getInstance()

            dd.currentLayer._widget:getChildByName("resinfo1"):getChildByName("detail"):getChildByName("info"):getChildByName("pg"):getChildByName("progress"):stopAllActions()
            dd.currentLayer._widget:getChildByName("resinfo1"):setVisible(false)
            dd.currentLayer._widget:getChildByName("resinfo2"):setVisible(false)
            dd.currentLayer._widget:getChildByName("resinfo3"):setVisible(false)

            pop.onClosePopDialog("colPop")
            dd.currentLayer._widget:showShare(widget._cache,false)
        end     
    end
end

--直接显示固定格子或者军队
function WorldControl:showGid(id,istroop)
    if istroop == false then
        self:closeAll()
        local index = self._worldUILayer._curMapLayer:getMapResIndex(id)
        if index >= 500 and index < 900 and index ~= 800 then
            return
        end

        local row = self._worldUILayer._curMapLayer:getMapData(id)
        if row ~= nil and row:rowCount() == 1 then
            data = {}
            data.cid = id
            data.type= row:getLong(1,"type")
            data.uid = row:getLong(1,"uid")
            data.level  = row:getLong(1,"level")
            data.avatar = row:getLong(1,"avatar")

            data.unionid = row:getLong(1,"unionid")
            data.power = row:getLong(1,"power")
            data.name = row:getString(1,"name")
            data.short = row:getString(1,"short")
            --cclog("1111111111111111111111111111111111111111")      
            if data.type > 20 and data.type < 30 then
                --cclog("===============4444444444444444===========2222222222222==========="..data.uid)
                if data.uid == 998 then
                    WorldControl:getInstance():showNpcPannelEx(data)
                else
                    --cclog("---------------------")
                    WorldControl:getInstance():showNpcPannel(data)
                end
            else
                self:showPannel(data,cc.p(0,0),false)
            end
        else
            local message = ff.net:createMessage()
            message:addCmd(0x0000004f)
            message:addLong(dd.uid)
            message:addLong(id)
            ff.net:rpc(message,false)            
        end        
        if row ~= nil then
            row:clear()
        end
    else
        local troop = ff.NetTable:create()
        troop:addCol("id",ff.NET_LONG)
        troop:addCol("uid",ff.NET_LONG)
        troop:addCol("state",ff.NET_LONG)
        troop:addCol("from",ff.NET_LONG)
        troop:addCol("to",ff.NET_LONG)
        troop:addCol("unionid",ff.NET_LONG)
        troop:addCol("type",ff.NET_LONG)
        troop:addCol("name",ff.NET_STRING,64)
        troop:addCol("short",ff.NET_STRING,16)

        for i=1,dd.myTroops:rowCount() do
            if dd.myTroops:getLong(i,"id") == id then
                troop:addRow()
                troop:setLong(1,"id",dd.myTroops:getLong(i,"id"))
                troop:setLong(1,"uid",dd.uid)
                troop:setLong(1,"state",dd.myTroops:getLong(i,"state"))
                troop:setLong(1,"type",dd.myTroops:getLong(i,"type"))
                troop:setLong(1,"from",dd.myTroops:getLong(i,"from"))
                troop:setLong(1,"to",dd.myTroops:getLong(i,"to"))
                troop:setLong(1,"unionid",dd.myUnion:getLong(1,"unionid"))
                troop:setString(1,"name",dd.myinfo:getString(1,"name"))
                troop:setString(1,"short",dd.myUnion:getString(1,"unionflag"))
                break
            end
        end
        if troop:rowCount() >= 1 then
            self:closeAll()
            --cclog("======11111"..troop:getLong(1,"state").."="..troop:getLong(1,"type"))
            if troop:getLong(1,"state") == 8 then
                self:onShowTroopEx(troop)
            else

                self:onShowTroop(troop)
            end
            troop:clear()
        end
    end
end

--打开世界地图
function runWorldScene(cid)
	--注册回退按钮
    local tick = ff.gameCore:getTick()
    dd.clearBackEvent()
    local scene        = cc.Director:getInstance():getRunningScene()
    local widget = scene:getChildByTag(1)
    if widget ~= nil then
        if widget == dd._cityLayer then
            dd._cityLayer:retain()
        end
        widget:stopAllActions()
        widget:removeFromParent(true)
    end
    local layer = WorldControl:getInstance()  
    layer._worldUILayer = createWorldUILayer()
    
    dd.clearScene()
    dd.worldLayer      = layer._worldUILayer
    dd.currentLayer    = layer._worldUILayer    
    dd.currentLayer._widget = dd.mainUI
    dd.currentLayer:initUI(cid)
    layer._worldData    = createMapData()
    layer._init = false
    layer:init()
    cclog("tick2:"..(ff.gameCore:getTick() - tick))
    scene:addChild(layer._worldUILayer,1,1)
    cclog("tick3:"..(ff.gameCore:getTick() - tick))
end

function WorldControl:getInstance()  
    if self.instance == nil then  
        self.instance = self:new()        
    end  
    return self.instance  
end  