--[[
世界地图地图显示层
]]
local WorldMapLayer = class("WorldMapLayer")
WorldMapLayer._name = "WorldMapLayer"

WorldMapLayer._serverId = 0
WorldMapLayer._schedulerID = 0

WorldMapLayer._tileWidth = 252
WorldMapLayer._tileHeight = 140
WorldMapLayer._clicksp = nil

function WorldMapLayer.extend(target)
    local t = tolua.getpeer(target)
    if not t then
        t = {}
        tolua.setpeer(target, t)
    end
    setmetatable(t, WorldMapLayer)
    return target
end

--根据地图内图块下标获取坐标 mapGid 转 mapPoint
function WorldMapLayer:getMapPointByMapGid(mapPos)
    cclog("map map row 1=========="..mapPos.x)
    cclog("map map col 1=========="..mapPos.y)
    local mapPoint = self:getMapPos(mapPos)
    --取出来是左下角，改到中间
    mapPoint = cc.pAdd(mapPoint, cc.p(self._tileWidth*self:getScale()/2, self._tileHeight*self:getScale()/2))
    cclog("map map row 2=========="..mapPos.x)
    cclog("map map col 2=========="..mapPos.y)
    cclog("map point=======x==="..mapPoint.x)
    cclog("map point=======y==="..mapPoint.y)
    return mapPoint
end

--xy=>cid => gid
--根据地图内坐标转化为地图内图块编号 屏幕坐标 =》服务端gid
function WorldMapLayer:getMapGidByMapScenePos(mapScenePos)
    --cid
    local mapId = self:inTilePos("base", mapScenePos)
    if mapId.x < 0 or mapId.x >= ff.world_maplayerSize.width  then
        return 0
    end
    if mapId.y < 0 or mapId.y >= ff.world_maplayerSize.height  then
        return 0
    end
    return mapId.x + mapId.y * ff.world_maplayerSize.width + 1
end

--根据屏幕坐标转化为大区内图块编号
--返回gid
function WorldMapLayer:getAreaGidByMapScenePos(mapScenePos)
	local mapGid = self:getMapGidByMapScenePos(mapScenePos)
    return mapGid
end

--根据屏幕坐标转化为大区坐标
function WorldMapLayer:getAreaPosByScenePos(scenePos)
	local mapWidth = self:getContentSize().width
	local mapHeight = self:getContentSize().height
	local serverRowId = self._serverId % 6
	local serverColId = math.floor(self._serverId / 6)
	local mapPos = self:convertToNodeSpace(scenePos)
	local areaPos = cc.pAdd(mapPos, cc.p(mapWidth*serverRowId, mapHeight*serverColId))
	return areaPos
end

--gid是服务端坐标
--imageid
--这里row.col是本地坐标系(255*1022)
function WorldMapLayer:changeGridByGid(gid, imageId)
    --坐标系转化
    local row = gid % ff.world_maplayerSize.width
    local col = math.floor((gid - row) / ff.world_maplayerSize.width)
    gid = col * ff.world_maplayerSize.width + row
end

function WorldMapLayer:addClick(x,y)
    if self._clicksp ~= nil then
        self._clicksp:stopAllActions()
    else
        self._clicksp = cc.Sprite:createWithSpriteFrameName("world/wclick.png")  
        self._clicksp:setOpacity(100)
        WorldControl:getInstance()._worldUILayer._mapContainer:addChild(self._clicksp,99,99)
    end
    self._clicksp:setVisible(true)
    self._clicksp:setPosition(cc.p(x*self:getScale(),y*self:getScale()))
end
function WorldMapLayer:removeClick()
    if self._clicksp ~= nil then
        self._clicksp:stopAllActions()
        local scale = self._clicksp:getScale()
        self._clicksp:runAction(cc.Sequence:create(cc.ScaleTo:create(0.05,scale - 0.05),cc.ScaleTo:create(0.125,scale),cc.RemoveSelf:create()))        
        self._clicksp = nil
    end
end



function WorldMapLayer:addTouchListener()
    local startp = nil
    local endp   = nil
    local function onTouchBegan(touch, event)
        --startp = touch:getLocation()
        startp = dd.changeTouchPos(touch:getLocation())
        local point = dd.changeTouchPos(touch:getLocation())
        point.y = point.y - 214

        local mapId = self:getMapGidByMapScenePos(point)
        if mapId <= 0 then
            return false
        end
        local mapPoint = self:getMapPosByCid(mapId)
        mapPoint = cc.pAdd(mapPoint, cc.p(self._tileWidth/2, self._tileHeight/2))
        local xx,yy = WorldControl:getInstance()._worldUILayer._mapContainer:getPosition()

        self:addClick(mapPoint.x,mapPoint.y)
        return true
    end
    local function onTouchCancel(touch,event)
        self:removeClick()
        hit = false
    end    
    local function onTouchMoved(touch, event)
        local p = dd.changeTouchPos(touch:getLocation())
        if math.abs(p.x - startp.x) <= 10 and math.abs(p.y - startp.y) <= 10 then
        else            
            WorldControl:getInstance()._worldUILayer._flowid = 0
        end
    end    

    local function onTouchEnded(touch, event)
        local point = dd.changeTouchPos(touch:getLocation())
        point.y = point.y - 214
        local touchGid = self:getAreaGidByMapScenePos(point)
        if touchGid <= 0 then
            self:removeClick()
            return
        end
        endp = dd.changeTouchPos(touch:getLocation())
        if math.abs(endp.x - startp.x) <= 10 and math.abs(endp.y - startp.y) <= 10 then
            local troop = self:isTouchTroops(self:convertToNodeSpace(endp))
            if troop == nil then
                local mapId = touchGid
                for i=1,dd.StrongholdCnt do
                    if WorldControl:getInstance()._worldData.Stronghold[i]["gid1"] == touchGid then
                        break
                    end
                    if WorldControl:getInstance()._worldData.Stronghold[i]["gid2"] == touchGid or WorldControl:getInstance()._worldData.Stronghold[i]["gid3"] == touchGid or WorldControl:getInstance()._worldData.Stronghold[i]["gid4"] == touchGid then
                        mapId = WorldControl:getInstance()._worldData.Stronghold[i]["gid1"]
                        break
                    end
                    if WorldControl:getInstance()._worldData.Stronghold[i]["gid5"] ~= nil then
                        if WorldControl:getInstance()._worldData.Stronghold[i]["gid5"] == touchGid or WorldControl:getInstance()._worldData.Stronghold[i]["gid6"] == touchGid or WorldControl:getInstance()._worldData.Stronghold[i]["gid7"] == touchGid or WorldControl:getInstance()._worldData.Stronghold[i]["gid8"] == touchGid or WorldControl:getInstance()._worldData.Stronghold[i]["gid9"] == touchGid then
                            mapId = WorldControl:getInstance()._worldData.Stronghold[i]["gid1"]
                            break
                        end
                    end
                end
                for i=1,4 do
                    if WorldControl:getInstance()._worldData.Stronghold2[i]["gid1"] == touchGid then
                        break
                    end
                    if WorldControl:getInstance()._worldData.Stronghold2[i]["gid2"] == touchGid or WorldControl:getInstance()._worldData.Stronghold2[i]["gid3"] == touchGid or WorldControl:getInstance()._worldData.Stronghold2[i]["gid4"] == touchGid then
                        mapId = WorldControl:getInstance()._worldData.Stronghold2[i]["gid1"]
                        break
                    end
                end
                --实际
                local mapPoint = self:getMapPosByCid(mapId)
                mapPoint = cc.pAdd(mapPoint, cc.p(self._tileWidth/2, self._tileHeight/2))
                
                --缩小后
                local xx,yy = WorldControl:getInstance()._worldUILayer._mapContainer:getPosition()
                --local fixx = (mapPoint.x + (xx - 360)/self:getScale())*self:getScale()
                --local fixy = (mapPoint.y + (yy - (ff.height - 214)/2)/self:getScale())*self:getScale()
                --zyj 2015-12-1 重新计算移动位置以适应网格
                local offx,offy
                if dd.openGrid then
                    offx = ff.width/2*dd.NodeGridTable.pro
                    offy = (ff.height/2)
                    local touch = {x = 0, y = offy}
                    touch = dd.changeTouchPos(touch)
                    --zyj 2015 
                    offy = touch.y - 214/2*1.2/dd.NodeGridTable.pro
                else
                    offx = ff.width/2
                    offy = (ff.height - 214)/2
                end
                local fixx = (mapPoint.x + (xx - offx)/self:getScale())*self:getScale()
                local fixy = (mapPoint.y + (yy - offy)/self:getScale())*self:getScale()

                local showimg = false
                self:showMapItem(touchGid,point,showimg)
                WorldControl:getInstance()._worldUILayer._mapContainer:runAction(cc.MoveBy:create(0.3, cc.p(-fixx,-fixy)))
            else
                WorldControl:getInstance():onShowTroop(troop)
                troop:clear()    
            end    
        end
        self:removeClick()
        --guide, auther:lhz
        --GuideControl:clickCallback()
        --end
    end    

    local listener = cc.EventListenerTouchOneByOne:create()
    listener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN )
    listener:registerScriptHandler(onTouchMoved,cc.Handler.EVENT_TOUCH_MOVED )
    listener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED )
    
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listener, self)
end

function WorldMapLayer:showMapItem(gid,point,showimg)
    local data = nil
    local row = self:getMapData(gid)
    if row ~= nil and row:rowCount() == 1 then
        data = {}
        data.cid = gid
        data.type= row:getLong(1,"type")
        data.uid = row:getLong(1,"uid")
        data.level  = row:getLong(1,"level")
        data.avatar = row:getLong(1,"avatar")
        data.unionid = row:getLong(1,"unionid")
        data.power = row:getLong(1,"power")
        data.name = row:getString(1,"name")
        data.short = row:getString(1,"short")
    end    
    if row ~= nil then
        row:clear()
    end        
    if data == nil then
        --问服务器获取
        data = {}
        data.cid = gid
        data.type= 0
        data.uid = 0
        data.level=0
        data.name=""
        data.short=""
        data.avatar=0
        data.unionid=0
        data.power = 0
        if WorldControl:getInstance()._worldData:checkStronghold(gid) ~= 0 then
            if WorldControl:getInstance()._worldData:checkStronghold(gid) == 3 then
                pop.onShowPopDialog("StrongholdInfoEx",WorldControl:getInstance()._worldData:getStrongholdId(gid))
            else
                WorldControl:getInstance():showStronghold(gid,WorldControl:getInstance()._worldData:checkStronghold(gid))
            end
        elseif WorldControl:getInstance()._worldData:checkStronghold2(gid) ~= 0 then
            WorldControl:getInstance():showStronghold2(gid,WorldControl:getInstance()._worldData:checkStronghold2(gid))
        elseif WorldControl:getInstance()._worldData:checkTower(gid) ~= 0 then
            WorldControl:getInstance():showTower(gid,WorldControl:getInstance()._worldData:checkTower(gid))
        elseif WorldControl:getInstance()._worldData:checkStrongSpace(gid) ~= 0 then
            WorldControl:getInstance():showStrongSpace(gid,WorldControl:getInstance()._worldData:checkTower(gid))
        else
            WorldControl:getInstance():showPannel(data,point,showimg)
        end
    else    
        if data.type > 20 and data.type < 30 then
            if data.uid == 998 then
                WorldControl:getInstance():showNpcPannelEx(data)
            else
                WorldControl:getInstance():showNpcPannel(data)
            end
        else
            WorldControl:getInstance():showPannel(data,point,showimg)
        end
    end    
end
                                   
function WorldMapLayer:init()
    self:setAnchorPoint(cc.p(0.5,0.5))
    self:setPosition(cc.p(self:getContentSize().width/2, self:getContentSize().height/2))
    self:addMapBackGroud("frame.jpg",cc.size(512,512))
    self:addTouchListener()
end

function createWorldMapLayer(serverId, offsetHeight)
    local target = nil
    local plist  = true

    local Appearance = dd.HolidayAppearance
    if Appearance ~= 1 and Appearance ~= 2 then
        Appearance = 0
    end
    if dd.KSRoom == 0 then
        target = WorldMapLayer.extend(ccui.fatalityTiledMap:create("images/world/map.tmx",Appearance,false))
    else
        target = WorldMapLayer.extend(ccui.fatalityTiledMap:create("images/world/map2.tmx",Appearance,false))
	end
    target._serverId = serverId
    target._offsetHeight = offsetHeight
	target:init()
	return target
end
