require "world/worldMapLayer"
--[[
世界地图UI层
]]

local WorldUILayer = class("WorldUILayer")
WorldUILayer._name = "WorldUILayer"

WorldUILayer._widget       = nil   --界面UI,按钮
WorldUILayer._mapContainer = nil   --地图容器，设置坐标移动地图
WorldUILayer._curMapLayer  = nil   --当前地图图层
WorldUILayer._signpost     = nil   --玩家城市距离标识

WorldUILayer._userCityScenePos = cc.p(0,0)  --玩家城市在屏幕上的坐标

WorldUILayer._areaId = 0           --上一次请求数据所在地块
WorldUILayer._lastID = 0           --上一次显示在cid控件上的文字
WorldUILayer._flowid = 0

WorldUILayer._init = false
WorldUILayer._rheight = 640
WorldUILayer._lastTick = 0
WorldUILayer._scroll   = nil
WorldUILayer._targetCid=0
WorldUILayer._info = {}
WorldUILayer._index = 1

function WorldUILayer.extend(target)
    local t = tolua.getpeer(target)
    if not t then
        t = {}
        tolua.setpeer(target, t)
    end
    setmetatable(t, WorldUILayer)
    return target
end

--根据大区编号获取大区内x,y坐标
function WorldUILayer:getAreaPosByGid(gid)
    local posX = gid % ff.world_maplayerSize.width
    local posY = math.floor((gid - posX) / ff.world_maplayerSize.width)
    return self._curMapLayer:getMapPointByMapGid(cc.p(posX, posY))
end

--大区内坐标转化为图内坐标
function WorldUILayer:getMapPosByAreaPos(areaPos, map)
    local mapPosX = areaPos.x - map:getContentSize().width
    local mapPosY = areaPos.y - map:getContentSize().height 
    return cc.p(mapPosX, mapPosY)
end

--根据两个大区编号获取相差格子距离
function WorldUILayer:getDistanceByGid(firstGid, secondGid)
    local firstRowId = firstGid % ff.world_maplayerSize.width
    local firstColId = math.floor(firstGid / ff.world_maplayerSize.width)
    local secondRowId = secondGid % ff.world_maplayerSize.width
    local secondColId = math.floor(secondGid / ff.world_maplayerSize.width)
    local dis = math.floor(cc.pGetDistance(cc.p(firstRowId, firstColId), cc.p(secondRowId, secondColId)))
    -- cclog("dis================="..dis)
    return dis
end

--获取两点之间的角度
function WorldUILayer:getStartAngle(startPos, endPos)
    cclog("start.Pos:"..startPos.x..",startPos.y"..","..endPos.x..","..endPos.y)
    local tenValue = (startPos.x - endPos.x) / (startPos.y - endPos.y)
    local radians = math.atan(tenValue)
    local angle = radians * 57.29577951
    if endPos.y <= startPos.y and startPos.x <= endPos.x then
        angle = angle - 180
    -- end
    elseif endPos.x <= startPos.x and endPos.y <= startPos.y then
        angle = angle + 180
    end
    return angle
end

--获取当前屏幕中心点的格子下标
function WorldUILayer:getSceneCenterGid()
    --local centerPos = cc.p(360 - self._scroll:getPositionX(), 640 - self._scroll:getPositionY())
    --zyj 适应网格
    local centerPos
    if not dd.openGrid then
        centerPos = cc.p(360, self._rheight/2)
    else
        offx = ff.width/2*dd.NodeGridTable.pro
        offy = (ff.height/2)
        local touch = {x = 0, y = offy}
        touch = dd.changeTouchPos(touch)
        offy = touch.y - 214/2*1.2/dd.NodeGridTable.pro
        centerPos = cc.p(offx, offy)--dd.changeTouchPos(cc.p(360, self._rheight/2))
    end

    local centerGid = self._curMapLayer:getAreaGidByMapScenePos(centerPos)
    return centerGid
end

--显示当前地图中心点距离玩家城市图块的标识
function WorldUILayer:showUserCitySignpost(from)
    local show  = self._curMapLayer:getRotate(from,dd.mycity:getLong(1,"cid"))
    if show == nil then
        return
    end
    if show:getLong(1,"show") == 1 then
        --self._signpost:setPosition(show:getLong(1,"x"),show:getLong(1,"y") + 220)
        --self._signpost:setPosition(100,300)

        local rate = show:getLong(1,"rate") - 90
        if (rate > 90 and rate <=180) or (rate > -180 and rate < -90) then
            self._signpost:getChildByName("dis"):setFlippedY(true)
            --self._signpost:getChildByName("label"):setRotation(180)
        else
            self._signpost:getChildByName("dis"):setFlippedY(false)
            --self._signpost:getChildByName("label"):setRotation(0)
        end

        cclog("rate:"..rate)
        self._signpost:getChildByName("dis"):setRotation(show:getLong(1,"rate")-90)
        --self._labSignpost:setString(show:getLong(1,"dis"))
        self._signpost:getChildByName("label"):setString(show:getLong(1,"dis"))
        self._signpost:setVisible(true)
        show:clear()
    else
        self._signpost:setVisible(false)
    end
end



--讲屏幕中心点移动到指定位置
function WorldUILayer:setSceneCenterByGid(gid,update)
    local gridData = WorldControl:getInstance():getMapDataByAreaGId(gid)
    local map = self._curMapLayer --测试用，需要根据服务器ID获取地图gridData.serverId
    cclog("xx:"..gridData.rowIndex..",yy:"..gridData.colIndex)
    local mapPoint = map:getMapPointByMapGid(cc.p(gridData.rowIndex, gridData.colIndex))
    cclog("xx:"..mapPoint.x..",yy:"..mapPoint.y)

    local ww = ff.width
    local hh = self._rheight
    cclog("scale:"..self._curMapLayer:getScale())
    --缩小后
    --zyj 2015 -12-2为了适应网格 往下移动一定像素
    local offx = 0
    local offy = 0
    if dd.openGrid then
        offy = (ff.height/2)
        local touch = {x = ww/2, y = offy}
        touch = dd.changeTouchPos(touch)
        --zyj 2015 如果以后y坐标又偏移就是这里的问题
        offy = touch.y - 214/2*1.2/dd.NodeGridTable.pro
        offx = touch.x
    else
        offx = ww/2
        offy = hh/2
    end
    self._mapContainer:setPosition(cc.p(-mapPoint.x + offx,
     -mapPoint.y + offy))
    if update == nil then
        self._areaId = gridData.areaId
    end
    self._flowid = 0
end

function WorldUILayer:getMapPointByGID(gid)
    local gridData = WorldControl:getInstance():getMapDataByAreaGId(gid)
    local map = self._curMapLayer 
    local mapPoint = map:getMapPointByMapGid(cc.p(gridData.rowIndex, gridData.colIndex))
    return mapPoint
end

--部队跟随
function WorldUILayer:setFlowTroop(troopid)
    self._flowid = troopid
    self._scroll:stopAutoScroll()
end

function WorldUILayer:update(dt)
    --记录当前区块用于判断是否重新请求数据
    if dd.KSRoom > 0 then
        if dd.KFRoomInfo ~= nil then
            if dd.KFRoomInfo:rowCount() > 0 then
                local index = dd.KFRoomInfo:rowCount()
                if dd.KFRoomInfo:getLong(index,"startime") < dd.getServerTime() then
                    
                    local time = dd.KFRoomInfo:getLong(index,"endtime") - dd.getServerTime()
                    if time > 0 then
                        if dd.mainUI ~= nil then
                            dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("kfpaihang"):getChildByName("time"):setString(ff.getLeftTime(time,":"))
                        end
                        
                    end
                end
            end
        end
    end

    local curCenterGid = self:getSceneCenterGid()
    if curCenterGid > 0 then
        if self._lastID ~= curCenterGid then
            dd.currentLayer._widget:setCidTips(curCenterGid)
            self:showUserCitySignpost(curCenterGid)
            self._lastID = curCenterGid
        end

        local curAreaId = WorldControl:getInstance():getMapDataByAreaGId(curCenterGid - 1).areaId

        if self._areaId ~= curAreaId then
            cclog("=============移动到其他区块================")
            cclog("pre area id========="..self._areaId)
            cclog("cur area id========="..curAreaId)
            self._areaId = curAreaId
            WorldControl:getInstance()._worldData:requestMapQuery(curCenterGid)
        end
        if dd.getServerTime() > (WorldControl:getInstance()._worldData._lastUpdate - 20) then
            cclog("续订")
            WorldControl:getInstance()._worldData._lastUpdate = dd.getServerTime() + 30
            WorldControl:getInstance()._worldData:requestMapQuery(curCenterGid)
        end
    end
    if self._flowid > 0 then
        local pos = self._curMapLayer:getFlowTroop(self._flowid)
        if pos.x > 0 and pos.y > 0 then
            self._mapContainer:setPosition(cc.p(-pos.x * self._curMapLayer:getScale() + ff.width/2,-pos.y * self._curMapLayer:getScale() + self._rheight/2))
        else
            for i=1,dd.myTroops:rowCount() do
                if dd.myTroops:getLong(i,"id") == self._flowid then
                    local to = dd.myTroops:getLong(i,"to")
                    dd.worldLayer:setSceneCenterByGid(to - 1,true)   
                    break
                end
            end
            self._flowid = 0
        end
    end

    if dd.myAlchemist ~= nil then
        if dd.mainUI ~= nil and dd.mainUI:getChildByName("top") ~= nil then
            dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("lianjinsuo"):loadTexture("images/effect/Alchemist/world_al.png",0)
            dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("lianjinsuo"):setVisible(true)
        end
    else
        if dd.mainUI ~= nil and dd.mainUI:getChildByName("top") ~= nil then
            dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("lianjinsuo"):setVisible(false)
        end
    end

    local showico = false
    if self._info ~= nil and next(self._info) ~= nil and table.getn(self._info) >= 1 and dd.myUnion ~= nil and  dd.myUnion:getLong(1,"unionid") > 0 then
        for i=1,table.getn(self._info) do
            if self._info[i]["unionid"] == dd.myUnion:getLong(1,"unionid") then
                showico = true
            end
        end
    end

    if showico == true then
        if dd.mainUI ~= nil and dd.mainUI:getChildByName("top") ~= nil then
            dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("judian"):setVisible(true)
            local  item = dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("judian"):getChildByTag(1)
            if item == nil then
                item  = sp.SkeletonAnimation:create("images/city/Stronghold.json", "images/city/Stronghold.atlas")
                dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("judian"):addChild(item,0,1)
                item:setPosition(cc.p(item:getContentSize().width/2,item:getContentSize().height/2))
                item:setAnimation(0,"Judian_play",true)
            end
        end
    else
        if dd.mainUI ~= nil and dd.mainUI:getChildByName("top") ~= nil then
            local  item = dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("judian"):getChildByTag(1)
            if item ~= nil then
                item:removeFromParent()
            end
            dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("judian"):setVisible(false)
        end
    end

end

function WorldUILayer:onEnter()
    --定位到玩家城市
    self._scroll:setInnerContainerSize(cc.size(self._curMapLayer:getContentSize().width*0.8,self._curMapLayer:getContentSize().height*0.8))
    self._curMapLayer:setScale(0.8)
    self._curMapLayer:setPosition(self._curMapLayer:getContentSize().width/2*0.8,self._curMapLayer:getContentSize().height/2*0.8)

    local tick = ff.gameCore:getTick()
    self:setSceneCenterByGid(self._targetCid - 1)   
    cclog("cid:"..dd.mycity:getLong(1,"cid"))
    --self:setSceneCenterByGid((320 - 1) + (320 - 1) * 640)

    self._widget = addCityUI(self)
    self._curMapLayer:start()
    ff.net:registerEvent(2,dd.Tjf_union_AllyQuit,self.onAllyQuit)
    ff.net:registerEvent(2,dd.Tjf_map_GetMyStrongholdLists,function (message)self:GetMyStrongholdLists(message)end)
    dd.mainUI._lastMusic = dd.getServerTime() + 5
    dd.playBackMusic(0,false)
    self:getMyStronghold()
    self._info = {}
    self._index = 1
end

function WorldUILayer:getMyStronghold(  )
    local message = ff.net:createMessage()
    message:addCmd(0x00000060)
    message:addLong(dd.uid)
    ff.net:rpc(message,false)
end


function WorldUILayer:GetMyStrongholdLists( message )

    local code = message:getLong()
    if code == 0 then
        self._index = 1
        self._info = {}
        local count = message:getLong()
        
        cclog("!!!!!!!!!!!!!!!!!!!!!!!!!!!!1"..count)
        for i=1,count do
            local id = message:getLong()
            local state = message:getLong()--0未开放1报名2争夺3保护
            local endTime = message:getLong()
            local unionid = message:getLong()
            local short = message:getString(3)
            local name = message:getString(15)
            local ico = message:getLong()
            
            message:getLong()
            message:getLong()
            message:getLong()
       
            self._info[i] = {}
            self._info[i]["id"] = id
            self._info[i]["unionid"] = unionid
            
            self._info[i]["state"] = state

            self._info[i]["endTime"] = endTime
            self._info[i]["short"] = short
            self._info[i]["name"] = name
            self._info[i]["ico"] = ico
        end 
    end
    message:clear()
    message = nil

end
function WorldUILayer.onAllyQuit(params)
    local code = params[1]
    local action = params[2]
    cclog("onAllyQuit:"..code..","..action)
    if code == 0 then
        local widget = pop.getDialog("union")
        if widget ~= nil then
            widget._messageid = {0,0}
            widget:onClose()
            pop.onClosePopDialog("union")
        end
        if dd.currentLayer._widget ~= nil then
            if action == 1 then
                dd.currentLayer._widget:onShowTips("\t"..ff.langString("您已经退出了联盟"))
            end    
        end
    else
        if dd.currentLayer._widget ~= nil then
           dd.currentLayer._widget:onShowTips("\t"..dd.errorCode(code))  
        end
    end
end 

--注册系统按钮和我定义的事件
function WorldUILayer:registrEvent()
end

function WorldUILayer:onStart()
    dd.yunOpen()
    WorldControl:getInstance()._worldUILayer._widget:onShow(false)
    local function starMap()
        self:scheduleUpdateWithPriorityLua(function (dt)self:update(dt) end, 0)
        --self:showMovePathByGid(1,250000,1)
    end    

    if self._targetCid > 0 then
        WorldControl:getInstance()._worldData:requestMapQuery(self._targetCid)
    else
        if dd.mycity:getLong(1, "cid") > 0 then
            WorldControl:getInstance()._worldData:requestMapQuery(dd.mycity:getLong(1, "cid"))
        end
    end
    performWithDelay(self,starMap,1)
    schedule(self, self._widget.updateTroopsQue,1)
end
function WorldUILayer:joinWorldComplete()
    local tick = ff.gameCore:getTick()
    dd.dealCallBack(2)
    --------------------------guide Armature, auther:lhz
    if GuideControl.curId == 815 then
        if self._mapContainer ~= nil then
            self._mapContainer:stopAllActions()
        end
        self:setSceneCenterByGid(self._targetCid - 1)   
    end
    -------------------------------------guide end
    GuideControl:GuideCheck()
    cclog("use:"..(ff.gameCore:getTick() - tick))
    self:initStronghold()
end

function WorldUILayer:initStronghold( )
    if dd.KSRoom == 0 then
        for i=1,dd.StrongholdCnt do
            if WorldControl:getInstance()._worldData.Stronghold[i] ~= nil then
                WorldControl:getInstance()._worldUILayer._curMapLayer:addEffect(WorldControl:getInstance()._worldData.Stronghold[i]["gid1"],0)
                WorldControl:getInstance()._worldUILayer._curMapLayer:addEffect(WorldControl:getInstance()._worldData.Stronghold[i]["gid1"],9000001)
            end
        end
        for i=1,4 do
            if WorldControl:getInstance()._worldData.Stronghold2[i] then
                WorldControl:getInstance()._worldUILayer._curMapLayer:addEffect(WorldControl:getInstance()._worldData.Stronghold2[i]["gid1"],0)
                WorldControl:getInstance()._worldUILayer._curMapLayer:addEffect(WorldControl:getInstance()._worldData.Stronghold2[i]["gid1"],9000002)
            end
        end
    else
        for i=1,dd.StrongholdCnt do
            if WorldControl:getInstance()._worldData.Stronghold[i] ~= nil then
                WorldControl:getInstance()._worldUILayer._curMapLayer:addEffect(WorldControl:getInstance()._worldData.Stronghold[i]["gid1"],0)
                local id  = 900010*10 + i
                WorldControl:getInstance()._worldUILayer._curMapLayer:addEffect(WorldControl:getInstance()._worldData.Stronghold[i]["gid1"],id)
            end
        end
    end
end

function WorldUILayer:releaseStronghold(  )
    for i=1,dd.StrongholdCnt do
        if WorldControl:getInstance()._worldData.Stronghold[i] ~= nil then
            WorldControl:getInstance()._worldUILayer._curMapLayer:addEffect(WorldControl:getInstance()._worldData.Stronghold[i]["gid1"],0)
        end
    end
    for i=1,4 do
        if WorldControl:getInstance()._worldData.Stronghold2[i] ~= nil then
            WorldControl:getInstance()._worldUILayer._curMapLayer:addEffect(WorldControl:getInstance()._worldData.Stronghold2[i]["gid1"],0)
        end
    end
end

function WorldUILayer:onExit()
    local tick = ff.gameCore:getTick()
    cclog("WorldUILayer:onExit")
    
    --guide, add by lhz
    if GuideControl._curGuideLayer ~= nil then
        GuideControl._curGuideLayer:removeFromParent()
        GuideControl._curGuideLayer = nil
    end
    --end


    --ff.gameCore:asyncRemoveSpriteFrames("images/world/world.plist")
    --ff.gameCore:asyncRemoveSpriteFrames("images/world/mapback.plist")
    --ff.gameCore:asyncRemoveSpriteFrames("images/world/worldeffects.plist")
    --ff.gameCore:asyncRemoveSpriteFrames("images/world/move.plist")
    --ff.gameCore:startAsyncRemoves()

    --处理那些只有地图上带的资源
    if dd.currentLayer._widget ~= nil then
        dd.currentLayer._widget:getChildByName("share"):getChildByName("info"):getChildByName("icon"):getChildByName("icon"):loadTexture(dd.getPathFromUI("public/alpha.png",1))
        dd.currentLayer._widget:getChildByName("share"):setVisible(false)
        dd.currentLayer._widget:getChildByName("resinfo1"):setVisible(false)
        dd.currentLayer._widget:getChildByName("resinfo2"):setVisible(false)
        dd.currentLayer._widget:getChildByName("resinfo3"):setVisible(false)
    end

    ff.net:removeEvent(2,dd.Tjf_union_AllyQuit)
    

    dd.mainUI:clearDangrous()   
    self:releaseStronghold()
    WorldControl:getInstance():closePannel()
    WorldControl:getInstance()._worldData:requestMapQuery(0)
    WorldControl:getInstance()._worldData:removeListener()
    self:removeAllChildren(true)
    self:unscheduleUpdate()
    WorldUILayer.instance = nil

    if ff.set_power <= 1 then
        local cache = cc.SpriteFrameCache:getInstance()
        cache:removeSpriteFramesFromFile("images/world/world.plist")
        cache:removeSpriteFramesFromFile("images/world/mapback.plist") 
        cclog("images/active_"..dd.HolidayAppearance.."/move.plist")
        if dd.HolidayAppearance == 1 or dd.HolidayAppearance == 2 then
            cache:removeSpriteFramesFromFile("images/active_"..dd.HolidayAppearance.."/move.plist") 
        else
            cache:removeSpriteFramesFromFile("images/world/move.plist") 
        end
        --cache:removeSpriteFramesFromFile("images/world/worldeffects.plist") 
    end    
end
function WorldUILayer:initMapLayer()
    local tick = ff.gameCore:getTick()

    --去黑边
    ff.gameCore:addImageFile("images/world/world.png")
    local cache = cc.SpriteFrameCache:getInstance()
    cache:addSpriteFrames("images/world/world.plist")
    cache:addSpriteFrames("images/world/mapback.plist")

    local scroll = ccui.fatalityScrollView:create()
    scroll:setBackGroundColorType(ccui.LayoutBackGroundColorType.none)
    scroll:setBounceEnabled(false)
    scroll:setDirection(ccui.ScrollViewDir.both)
    self._rheight = ff.height - 214
    scroll:setContentSize(cc.size(ff.width,self._rheight))
    scroll:setPosition(cc.p(0,214))
    --调整地图滚动速度的参数
    scroll:setAutoSpeed(1.2,-600,1000)

    --self:addChild(scroll)
    dd.add3DNodeGrid(self, scroll)

    self._mapContainer = scroll:getInnerContainer()
    self._scroll = scroll

    cclog("cid:"..dd.mycity:getLong(1,"cid"))
    --获取玩家当前服务器地图
    local gridData = WorldControl:getInstance():getMapDataByAreaGId(dd.mycity:getLong(1, "cid") - 1)
    local map = createWorldMapLayer(1,0)
    self._mapContainer:addChild(map)

    cclog("ww:"..map:getContentSize().width..",hh:"..map:getContentSize().height.."=="..ff.world_maplayerSize.width)
    scroll:setInnerContainerSize(cc.size(map:getContentSize().width, map:getContentSize().height))
    scroll:setZoom(0.7,1.2,0.6,1.3)
    --设置边缘
    scroll:setBoundary(ff.width/2,-ff.width/2,-self._rheight/2,self._rheight/2)
    --设置为iso格式
    scroll:setScrollISO()
    self._curMapLayer = map
    
end

function WorldUILayer:initSignpost()
    local function goHome(sender,eventType)
        if eventType == ccui.TouchEventType.ended then
            self._flowid = 0
            self:setSceneCenterByGid(dd.mycity:getLong(1,"cid") - 1,true)   
        end
    end
    --dis_tag
    self._signpost = dd.currentLayer._widget:getChildByName("dis_tag"):clone()
    self._signpost:setPosition(ff.width - 80,ff.height - 260)

    self._signpost:setVisible(false)
    self:addChild(self._signpost, 1)
    self._signpost:addTouchEventListener(goHome)
end


function WorldUILayer:initUI(cid)
    if self._init == false then
        --ff.dbmanager:sql_query("delete from client_map")
        --ff.dbmanager:sql_query("delete from client_map_user")
        self._lastTick = ff.gameCore:getTick()
        self._init = true
    end
    if dd.KSRoom == 0 then
        self._rheight = 640
        if dd.mainUI ~= nil then 
            dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("col"):setVisible(true)

            dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("kfpaihang"):setVisible(false)
        end
    else
        self._rheight = 200
        if dd.mainUI ~= nil then
            dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("col"):setVisible(false)
            dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("kfpaihang"):setVisible(true)
            dd.mainUI:getChildByName("top"):getChildByName("topworld"):getChildByName("kfpaihang"):getChildByName("Image_793"):loadTexture("images/active/kfpaihang.png",0)

        end
    end
    
    self._flowid = 0
    self:registrEvent()
    self:initMapLayer()
    self:initSignpost()

    if cid ~= nil then
        self._targetCid = cid
    else
        self._targetCid = dd.mycity:getLong(1,"cid")
    end

    if dd.mainUI ~= nil then
        dd.mainUI:getChildByName("mask"):setVisible(false)
    end

    --显示阶段    
    local function onEnterOrExit(tag)
        cclog(tag)
        if tag == "enter" then
            local tick = ff.gameCore:getTick()
            self:onEnter()
            cclog("w enter:"..(ff.gameCore:getTick() - tick))
        elseif tag == "enterTransitionFinish" then
            local tick = ff.gameCore:getTick()
            self:onStart()
            cclog("w enterTransitionFinish:"..(ff.gameCore:getTick() - tick))
        elseif tag == "exitTransitionStart" then   
        elseif tag == "exit" then    
            local tick = ff.gameCore:getTick()
            self:onExit()
            cclog("out world:"..(ff.gameCore:getTick() - tick))
        elseif tag == "cleanup" then
            if ff.set_power <= 1 then
                cc.Director:getInstance():getTextureCache():removeUnusedTextures()     
            end
        end         
    end
    self:registerScriptHandler(onEnterOrExit)

end

function WorldUILayer:getCid(gid)
    local row = gid % ff.world_maplayerSize.width
    local col = math.floor((gid - row) / ff.world_maplayerSize.width)
    local cid = col * ff.world_maplayerSize.width + row
    return cid
end

--播放收兵动画
function WorldUILayer:playGuideArmature()
    local function checkGuide()
        GuideControl:QueryMinePos()
    end
    local xx = ff.width/2
    local yy = self._rheight/2 + 214
    local frames = {}
    for i=1,6 do
        frames[i] = {}
        local count = 12
        if i == 6 then
            count = 6
        end
        for k=1,5 do
            frames[i][k] = {}
            for j = 1, count do
                local frame = cc.SpriteFrameCache:getInstance():getSpriteFrame("troop_"..i.."_"..k.."000"..j..".png")
                frames[i][k][j] = frame
            end
        end
    end
    --固定出生点
    local pos  = {{cc.p(360, 1340),2}, {cc.p(855, 1135),3}, {cc.p(1060, 640),4}, {cc.p(855, 145),5}, {cc.p(360, -60),1}, {cc.p(-130, 145),5}, {cc.p(-340, 640),4}, {cc.p(-130, 1135),3}}
    --生成动画
    local maxtime = 0
    for i=1,30 do
        --选兵种
        local index = math.floor(math.random()*99999)%5 + 1
        local dir   = pos[i%8 + 1][2]
        local sprite =  cc.Sprite:create()
        sprite:setContentSize(cc.size(128,128))
        sprite:setAnchorPoint(cc.p(0.5,0.5))
        local animation = cc.Animation:createWithSpriteFrames(frames[index][dir],0.1)
        sprite:runAction(cc.RepeatForever:create(cc.Animate:create(animation)))
        sprite:setPosition(pos[i%8 + 1][1])
        if i%8 + 1 >= 6 then
            sprite:setFlippedX(true)
        end
        local tick = math.random()*3
        local action = cc.Sequence:create(
                    cc.MoveTo:create(2 + tick, cc.p(xx, yy)),
                    cc.FadeOut:create(0.3),
                    cc.RemoveSelf:create()
                    )
        sprite:runAction(action)
        dd.currentLayer:addChild(sprite,math.floor((3 - tick) * 1000))
        if tick > maxtime then
            maxtime = tick
        end
    end
    performWithDelay(self, checkGuide, maxtime + 2)
end


function createWorldUILayer()
	local target = WorldUILayer.extend(cc.Layer:create())	
	return target
end